from collections.abc import Sequence
from enum import Enum
from typing import Any, ClassVar

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.migrations.state import ProjectState
from django.db.models import Model
from typing_extensions import Self

class OperationCategory(str, Enum):
    ADDITION = "+"
    REMOVAL = "-"
    ALTERATION = "~"
    PYTHON = "p"
    SQL = "s"
    MIXED = "?"

class Operation:
    reversible: ClassVar[bool]
    reduces_to_sql: ClassVar[bool]
    atomic: ClassVar[bool]
    elidable: ClassVar[bool]
    serialization_expand_args: ClassVar[list[str]]
    category: ClassVar[OperationCategory | None]
    def __new__(cls, *args: Any, **kwargs: Any) -> Self: ...
    def deconstruct(self) -> tuple[str, Sequence[Any], dict[str, Any]]: ...
    def state_forwards(self, app_label: str, state: ProjectState) -> None: ...
    def database_forwards(
        self, app_label: str, schema_editor: BaseDatabaseSchemaEditor, from_state: ProjectState, to_state: ProjectState
    ) -> None: ...
    def database_backwards(
        self, app_label: str, schema_editor: BaseDatabaseSchemaEditor, from_state: ProjectState, to_state: ProjectState
    ) -> None: ...
    def describe(self) -> str: ...
    def formatted_description(self) -> str: ...
    @property
    def migration_name_fragment(self) -> str: ...
    def references_model(self, name: str, app_label: str) -> bool: ...
    def references_field(self, model_name: str, name: str, app_label: str) -> bool: ...
    def allow_migrate_model(self, connection_alias: BaseDatabaseWrapper | str, model: type[Model]) -> bool: ...
    def reduce(self, operation: Operation, app_label: str) -> bool | list[Operation]: ...
    def __replace__(self, /, **changes: Any) -> Self: ...



from typing import Any

from django.core.mail.backends.console import EmailBackend as ConsoleEmailBackend

class EmailBackend(ConsoleEmailBackend):
    file_path: str
    def __init__(self, *args: Any, file_path: str | None = None, **kwargs: Any) -> None: ...



from collections.abc import Iterator
from typing import IO, Any, AnyStr, Generic

from django.utils._os import _PathCompatible

def validate_file_name(name: _PathCompatible, allow_relative_path: bool = False) -> _PathCompatible: ...

class FileProxyMixin(Generic[AnyStr]):
    file: IO[AnyStr] | None
    @property
    def encoding(self) -> Any: ...
    @property
    def fileno(self) -> Any: ...
    @property
    def flush(self) -> Any: ...
    @property
    def isatty(self) -> Any: ...
    @property
    def newlines(self) -> Any: ...
    @property
    def read(self) -> Any: ...
    @property
    def readinto(self) -> Any: ...
    @property
    def readline(self) -> Any: ...
    @property
    def readlines(self) -> Any: ...
    @property
    def seek(self) -> Any: ...
    @property
    def tell(self) -> Any: ...
    @property
    def truncate(self) -> Any: ...
    @property
    def write(self) -> Any: ...
    @property
    def writelines(self) -> Any: ...
    @property
    def closed(self) -> bool: ...
    def readable(self) -> bool: ...
    def writable(self) -> bool: ...
    def seekable(self) -> bool: ...
    def __iter__(self) -> Iterator[AnyStr]: ...



from django.contrib.sessions.backends.base import SessionBase
from typing_extensions import override

class SessionStore(SessionBase):
    @override
    def exists(self, session_key: str | None = None) -> bool: ...
    @override
    async def aexists(self, session_key: str | None = None) -> bool: ...



from types import ModuleType
from typing import Any

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.backends.utils import CursorDebugWrapper as BaseCursorDebugWrapper
from django.db.backends.utils import _ExecuteQuery
from django.utils.functional import cached_property

from .client import DatabaseClient
from .creation import DatabaseCreation
from .features import DatabaseFeatures
from .introspection import DatabaseIntrospection
from .operations import DatabaseOperations

TIMESTAMPTZ_OID: int

def psycopg_version() -> tuple[int, int, int]: ...

class DatabaseWrapper(BaseDatabaseWrapper):
    client: DatabaseClient
    creation: DatabaseCreation
    features: DatabaseFeatures
    introspection: DatabaseIntrospection
    ops: DatabaseOperations

    client_class: type[DatabaseClient]
    creation_class: type[DatabaseCreation]
    features_class: type[DatabaseFeatures]
    introspection_class: type[DatabaseIntrospection]
    ops_class: type[DatabaseOperations]

    Database: ModuleType
    operators: dict[str, str]
    pattern_esc: str
    pattern_ops: dict[str, str]

    # PostgreSQL backend-specific attributes.
    _named_cursor_idx: int
    @property
    def pool(self) -> Any: ...
    def close_pool(self) -> None: ...
    def tzinfo_factory(self, offset: int) -> Any: ...
    @cached_property
    def pg_version(self) -> int: ...

class CursorMixin:
    def callproc(self, name: Any, args: Any = ...) -> Any: ...

class ServerBindingCursor(CursorMixin): ...
class Cursor(CursorMixin): ...
class ServerSideCursor(CursorMixin): ...

class CursorDebugWrapper(BaseCursorDebugWrapper):
    def copy(self, statement: _ExecuteQuery) -> Any: ...



from django.db.backends.base.features import BaseDatabaseFeatures
from django.db.backends.sqlite3.base import DatabaseWrapper
from django.utils.functional import cached_property
from typing_extensions import override

class DatabaseFeatures(BaseDatabaseFeatures):
    connection: DatabaseWrapper
    can_alter_table_drop_column: bool
    supports_aggregate_order_by_clause: bool
    supports_aggregate_distinct_multiple_argument: bool
    supports_any_value: bool
    @cached_property
    @override
    def django_test_skips(self) -> dict[str, set[str]]: ...  # type: ignore[override]
    @cached_property
    @override
    def introspected_field_types(self) -> dict[str, str]: ...  # type: ignore[override]
    @property
    @override
    def max_query_params(self) -> int: ...  # type: ignore[override]
    @cached_property
    @override
    def supports_json_field(self) -> bool: ...  # type: ignore[override]
    @property
    @override
    def can_introspect_json_field(self) -> bool: ...  # type: ignore[override]
    @property
    @override
    def has_json_object_function(self) -> bool: ...  # type: ignore[override]
    @cached_property
    @override
    def can_return_columns_from_insert(self) -> bool: ...  # type: ignore[override]
    @property
    @override
    def can_return_rows_from_bulk_insert(self) -> bool: ...  # type: ignore[override]
    @property
    @override
    def can_return_rows_from_update(self) -> bool: ...  # type: ignore[override]



from collections.abc import Sequence
from typing import Any, ClassVar

from django.db.models.expressions import BaseExpression, Combinable
from django.db.models.query import _OrderByFieldName
from typing_extensions import override

class OrderableAggMixin:
    allow_order_by: ClassVar[bool]
    def __init__(
        self,
        *expressions: BaseExpression | Combinable | str,
        ordering: Sequence[_OrderByFieldName] = ...,
        order_by: Sequence[_OrderByFieldName] = ...,
        **extra: Any,
    ) -> None: ...
    @override
    def __init_subclass__(cls, *args: Any, **kwargs: Any) -> None: ...



from typing import Any

from django.db.models.sql.compiler import SQLAggregateCompiler as SQLAggregateCompiler
from django.db.models.sql.compiler import SQLCompiler as BaseSQLCompiler
from django.db.models.sql.compiler import SQLDeleteCompiler as SQLDeleteCompiler
from django.db.models.sql.compiler import SQLInsertCompiler as BaseSQLInsertCompiler
from django.db.models.sql.compiler import SQLUpdateCompiler as SQLUpdateCompiler
from typing_extensions import override

__all__ = [
    "SQLAggregateCompiler",
    "SQLCompiler",
    "SQLDeleteCompiler",
    "SQLInsertCompiler",
    "SQLUpdateCompiler",
]

class InsertUnnest(list[str]): ...

class SQLCompiler(BaseSQLCompiler):
    @override
    def quote_name_unless_alias(self, name: str) -> str: ...

class SQLInsertCompiler(BaseSQLInsertCompiler):
    @override
    def assemble_as_sql(self, fields: Any, value_rows: Any) -> tuple[Any, Any]: ...



import re
from typing import Any, ClassVar, TypeAlias

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models import Expression, Field, FloatField, TextField
from django.db.models.expressions import Combinable, CombinedExpression, Func, Value
from django.db.models.lookups import Lookup
from django.db.models.sql.compiler import SQLCompiler, _AsSqlType
from psycopg.adapt import Dumper
from typing_extensions import Self, override

from .utils import CheckPostgresInstalledMixin

_Expression: TypeAlias = str | Combinable | SearchQueryCombinable

class UTF8Dumper(Dumper):
    @override
    def dump(self, obj: object) -> bytes: ...

def quote_lexeme(value: str) -> str: ...

spec_chars_re: re.Pattern[str]
multiple_spaces_re: re.Pattern[str]

def normalize_spaces(val: str) -> str | None: ...
def psql_escape(query: str) -> str | None: ...

class SearchVectorExact(Lookup):
    @override
    def process_rhs(self, qn: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
    @override
    def as_sql(self, qn: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...

class SearchVectorField(CheckPostgresInstalledMixin, Field): ...
class SearchQueryField(CheckPostgresInstalledMixin, Field): ...

class SearchConfig(Expression):
    config: _Expression | None
    def __init__(self, config: _Expression) -> None: ...
    @classmethod
    def from_parameter(cls, config: _Expression | None) -> SearchConfig: ...

class SearchVectorCombinable:
    ADD: str

class SearchVector(SearchVectorCombinable, Func):
    config: _Expression | None
    function: str
    arg_joiner: str
    output_field: ClassVar[SearchVectorField]
    def __init__(
        self, *expressions: _Expression, config: _Expression | None = None, weight: Any | None = None
    ) -> None: ...
    @override
    def as_sql(  # type: ignore[override]
        self,
        compiler: SQLCompiler,
        connection: BaseDatabaseWrapper,
        function: str | None = None,
        template: str | None = None,
    ) -> _AsSqlType: ...

class CombinedSearchVector(SearchVectorCombinable, CombinedExpression):
    def __init__(
        self,
        lhs: Combinable,
        connector: str,
        rhs: Combinable,
        config: _Expression | None,
        output_field: Field | None = None,
    ) -> None: ...

class SearchQueryCombinable:
    BITAND: str
    BITOR: str
    def __or__(self, other: SearchQueryCombinable) -> Self: ...
    def __ror__(self, other: SearchQueryCombinable) -> Self: ...
    def __and__(self, other: SearchQueryCombinable) -> Self: ...
    def __rand__(self, other: SearchQueryCombinable) -> Self: ...

class SearchQuery(SearchQueryCombinable, Func):  # type: ignore[misc]
    output_field: ClassVar[SearchQueryField]
    SEARCH_TYPES: dict[str, str]
    def __init__(
        self,
        value: _Expression,
        output_field: Field | None = None,
        *,
        config: _Expression | None = None,
        invert: bool = False,
        search_type: str = "plain",
    ) -> None: ...
    @override
    def as_sql(  # type: ignore[override]
        self,
        compiler: SQLCompiler,
        connection: BaseDatabaseWrapper,
        function: str | None = None,
        template: str | None = None,
    ) -> _AsSqlType: ...
    @override
    def __invert__(self) -> Self: ...  # type: ignore[override]

class CombinedSearchQuery(SearchQueryCombinable, CombinedExpression):  # type: ignore[misc]
    def __init__(
        self,
        lhs: Combinable,
        connector: str,
        rhs: Combinable,
        config: _Expression | None,
        output_field: Field | None = None,
    ) -> None: ...

class SearchRank(Func):
    output_field: ClassVar[FloatField]
    def __init__(
        self,
        vector: SearchVector | _Expression,
        query: SearchQuery | _Expression,
        weights: Any | None = None,
        normalization: Any | None = None,
        cover_density: bool = False,
    ) -> None: ...

class SearchHeadline(Func):
    function: str
    template: str
    output_field: ClassVar[TextField]
    def __init__(
        self,
        expression: _Expression,
        query: _Expression,
        *,
        config: _Expression | None = None,
        start_sel: Any | None = None,
        stop_sel: Any | None = None,
        max_words: int | None = None,
        min_words: int | None = None,
        short_word: str | None = None,
        highlight_all: bool | None = None,
        max_fragments: int | None = None,
        fragment_delimiter: str | None = None,
    ) -> None: ...
    @override
    def as_sql(  # type: ignore[override]
        self,
        compiler: SQLCompiler,
        connection: BaseDatabaseWrapper,
        function: str | None = None,
        template: str | None = None,
    ) -> _AsSqlType: ...

class TrigramBase(Func):
    output_field: ClassVar[FloatField]
    def __init__(self, expression: _Expression, string: str, **extra: Any) -> None: ...

class TrigramWordBase(Func):
    output_field: ClassVar[FloatField]
    def __init__(self, string: str, expression: _Expression, **extra: Any) -> None: ...

class TrigramSimilarity(TrigramBase): ...
class TrigramDistance(TrigramBase): ...
class TrigramWordDistance(TrigramWordBase): ...
class TrigramStrictWordDistance(TrigramWordBase): ...
class TrigramWordSimilarity(TrigramWordBase): ...
class TrigramStrictWordSimilarity(TrigramWordBase): ...

class LexemeCombinable:
    BITAND: str
    BITOR: str
    def __or__(self, other: LexemeCombinable) -> CombinedLexeme: ...
    def __ror__(self, other: LexemeCombinable) -> CombinedLexeme: ...
    def __and__(self, other: LexemeCombinable) -> CombinedLexeme: ...
    def __rand__(self, other: LexemeCombinable) -> CombinedLexeme: ...

class Lexeme(LexemeCombinable, Value):  # type: ignore[misc]
    def __init__(
        self,
        value: Any,
        output_field: Any | None = None,
        *,
        invert: bool = False,
        prefix: bool = False,
        weight: Any | None = None,
    ) -> None: ...
    @override
    def __invert__(self) -> Self: ...  # type: ignore[override]

class CombinedLexeme(LexemeCombinable, CombinedExpression):  # type: ignore[misc]
    @override
    def __invert__(self) -> Self: ...  # type: ignore[override]



from typing import Any, TypeVar

from django.contrib.admin import ModelAdmin
from django.contrib.gis.forms import OSMWidget
from django.db.models import Model
from django.db.models.fields import Field
from django.forms.fields import Field as FormField
from django.http.request import HttpRequest

_ModelT = TypeVar("_ModelT", bound=Model)

class GeoModelAdminMixin:
    gis_widget: type[OSMWidget]
    gis_widget_kwargs: dict[str, Any]
    def formfield_for_dbfield(self, db_field: Field, request: HttpRequest, **kwargs: Any) -> FormField | None: ...

class GISModelAdmin(GeoModelAdminMixin, ModelAdmin[_ModelT]): ...



from collections.abc import Iterable
from typing import Any

from django.core.serializers import base
from django.db.models.base import Model

class Serializer(base.Serializer):
    objects: list[Any]
    def get_dump_object(self, obj: Model) -> dict[str, Any]: ...

class Deserializer(base.Deserializer):
    def __init__(
        self,
        object_list: Iterable[dict[str, Any]],
        *,
        using: str = ...,
        ignorenonexistent: bool = False,
        **options: Any,
    ) -> None: ...



import functools
import sys
import threading
from collections.abc import Callable
from contextlib import AbstractContextManager
from inspect import Signature, _ParameterKind
from types import FunctionType
from typing import Any, TypeGuard

ARG_KINDS: frozenset[_ParameterKind]

if sys.version_info >= (3, 14):
    lock: threading.Lock
    safe_signature_from_callable: functools.partial[Signature]

def get_func_args(func: Callable[..., Any]) -> list[str]: ...
def get_func_full_args(func: Callable[..., Any]) -> list[tuple[str, str] | tuple[str]]: ...
def func_accepts_kwargs(func: Callable[..., Any]) -> bool: ...
def func_accepts_var_args(func: Callable[..., Any]) -> bool: ...
def method_has_no_args(meth: Callable[..., Any]) -> bool: ...
def func_supports_parameter(func: Callable[..., Any], name: str) -> bool: ...
def is_module_level_function(func: Callable[..., Any]) -> TypeGuard[FunctionType]: ...
def lazy_annotations() -> AbstractContextManager[None]: ...



from collections.abc import AsyncIterable, AsyncIterator, Awaitable, Callable, Iterable, Iterator, Mapping
from http import HTTPStatus
from http.cookies import SimpleCookie
from io import BytesIO, IOBase
from json import JSONEncoder
from re import Pattern
from types import TracebackType
from typing import Any, Generic, Literal, NoReturn, TypeAlias, TypedDict, TypeVar, type_check_only

from asgiref.typing import ASGIVersions
from django.contrib.auth.models import _User
from django.contrib.sessions.backends.base import SessionBase
from django.core.handlers.asgi import ASGIRequest
from django.core.handlers.base import BaseHandler
from django.core.handlers.wsgi import WSGIRequest
from django.http.request import HttpRequest
from django.http.response import HttpResponseBase
from django.template.base import Template
from django.test.utils import ContextList
from django.urls import ResolverMatch
from django.utils.functional import _StrOrPromise, cached_property
from typing_extensions import override

BOUNDARY: str
MULTIPART_CONTENT: str
CONTENT_TYPE_RE: Pattern[str]
JSON_CONTENT_TYPE_RE: Pattern[str]
REDIRECT_STATUS_CODES: frozenset[HTTPStatus]

class RedirectCycleError(Exception):
    last_response: HttpResponseBase
    redirect_chain: list[tuple[str, int]]
    def __init__(self, message: str, last_response: HttpResponseBase) -> None: ...

class FakePayload(IOBase):
    read_started: bool
    def __init__(self, initial_bytes: bytes | str | None = ...) -> None: ...
    def __len__(self) -> int: ...
    @override
    def read(self, size: int = ..., /) -> bytes: ...
    @override
    def readline(self, size: int | None = ..., /) -> bytes: ...
    @override
    def write(self, content: bytes | str, /) -> None: ...

_T = TypeVar("_T")

def closing_iterator_wrapper(iterable: Iterable[_T], close: Callable[[], Any]) -> Iterator[_T]: ...
async def aclosing_iterator_wrapper(iterable: AsyncIterable[_T], close: Callable[[], Any]) -> AsyncIterator[_T]: ...
def conditional_content_removal(request: HttpRequest, response: HttpResponseBase) -> HttpResponseBase: ...

@type_check_only
class _WSGIResponse(HttpResponseBase):
    wsgi_request: WSGIRequest

@type_check_only
class _ASGIResponse(HttpResponseBase):
    asgi_request: ASGIRequest

class ClientHandler(BaseHandler):
    enforce_csrf_checks: bool
    def __init__(self, enforce_csrf_checks: bool = ..., *args: Any, **kwargs: Any) -> None: ...
    def __call__(self, environ: dict[str, Any]) -> _WSGIResponse: ...

class AsyncClientHandler(BaseHandler):
    enforce_csrf_checks: bool
    def __init__(self, enforce_csrf_checks: bool = ..., *args: Any, **kwargs: Any) -> None: ...
    async def __call__(self, scope: dict[str, Any]) -> _ASGIResponse: ...

def encode_multipart(boundary: str, data: dict[str, Any]) -> bytes: ...
def encode_file(boundary: str, key: str, file: Any) -> list[bytes]: ...

_GetDataType: TypeAlias = (
    Mapping[str, str | bytes | int | Iterable[str | bytes | int]]
    | Iterable[tuple[str, str | bytes | int | Iterable[str | bytes | int]]]
    | None
)

@type_check_only
class _RequestFactory(Generic[_T]):
    json_encoder: type[JSONEncoder]
    defaults: dict[str, str]
    cookies: SimpleCookie
    errors: BytesIO
    def __init__(
        self,
        *,
        json_encoder: type[JSONEncoder] = ...,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **defaults: Any,
    ) -> None: ...
    def request(self, **request: Any) -> _T: ...
    def get(
        self,
        path: _StrOrPromise,
        data: _GetDataType = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _T: ...
    def post(
        self,
        path: _StrOrPromise,
        data: Any = ...,
        content_type: str = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _T: ...
    def head(
        self,
        path: _StrOrPromise,
        data: Any = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _T: ...
    def trace(
        self,
        path: _StrOrPromise,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _T: ...
    def options(
        self,
        path: _StrOrPromise,
        data: dict[str, str] | str = ...,
        content_type: str = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _T: ...
    def put(
        self,
        path: _StrOrPromise,
        data: Any = ...,
        content_type: str = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _T: ...
    def patch(
        self,
        path: _StrOrPromise,
        data: Any = ...,
        content_type: str = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _T: ...
    def delete(
        self,
        path: _StrOrPromise,
        data: Any = ...,
        content_type: str = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _T: ...
    def generic(
        self,
        method: str,
        path: _StrOrPromise,
        data: Any = ...,
        content_type: str | None = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _T: ...

class RequestFactory(_RequestFactory[WSGIRequest]): ...

# A non total duplication of `asgiref.typing.HTTPScope`
@type_check_only
class _HTTPScope(TypedDict, total=False):
    type: Literal["http"]
    asgi: ASGIVersions
    http_version: str
    method: str
    scheme: str
    path: str
    raw_path: bytes
    query_string: bytes
    root_path: str
    headers: Iterable[tuple[bytes, bytes]]
    client: tuple[str, int] | None
    server: tuple[str, int | None] | None
    state: dict[str, Any]
    extensions: dict[str, dict[object, object]] | None

@type_check_only
class _AsyncRequestFactory(_RequestFactory[_T]):
    defaults: _HTTPScope  # type: ignore[assignment]

class AsyncRequestFactory(_AsyncRequestFactory[ASGIRequest]): ...

# fakes to distinguish WSGIRequest and ASGIRequest
@type_check_only
class _MonkeyPatchedWSGIResponse(_WSGIResponse):
    def json(self) -> Any: ...
    request: dict[str, Any]
    client: Client
    templates: list[Template]

    # `context` and `context_data` are populated based on whether you use the standard template
    # backend or a custom one respectively.
    # `context_data` only exists if the response was successful too as the return type changes.
    # `HTTPResponse` when API failed and `TemplateResponse` when successful.
    # https://docs.djangoproject.com/en/stable/topics/testing/tools/#django.test.Response.context
    context: ContextList | dict[str, Any]
    # Must match `django.template.response.SimpleTemplateResponse.context_data`
    context_data: dict[str, Any] | None

    content: bytes
    resolver_match: ResolverMatch
    redirect_chain: list[tuple[str, int]]
    @cached_property
    def text(self) -> str: ...

@type_check_only
class _MonkeyPatchedASGIResponse(_ASGIResponse):
    def json(self) -> Any: ...
    request: dict[str, Any]
    client: AsyncClient
    templates: list[Template]

    # `context` and `context_data` are populated based on whether you use the standard template
    # backend or a custom one respectively.
    # `context_data` only exists if the response was successful too as the return type changes.
    # `HTTPResponse` when API failed and `TemplateResponse` when successful.
    # https://docs.djangoproject.com/en/stable/topics/testing/tools/#django.test.Response.context
    context: ContextList | dict[str, Any]
    # Must match `django.template.response.SimpleTemplateResponse.context_data`
    context_data: dict[str, Any] | None

    content: bytes
    resolver_match: ResolverMatch
    redirect_chain: list[tuple[str, int]]

class ClientMixin:
    def store_exc_info(self, **kwargs: Any) -> None: ...
    def check_exception(self, response: HttpResponseBase) -> NoReturn: ...
    @property
    def session(self) -> SessionBase: ...
    async def asession(self) -> SessionBase: ...
    def login(self, **credentials: Any) -> bool: ...
    async def alogin(self, **credentials: Any) -> bool: ...
    def force_login(self, user: _User, backend: str | None = ...) -> None: ...
    async def aforce_login(self, user: _User, backend: str | None = ...) -> None: ...
    def logout(self) -> None: ...
    async def alogout(self) -> None: ...

class Client(ClientMixin, _RequestFactory[_MonkeyPatchedWSGIResponse]):
    handler: ClientHandler
    raise_request_exception: bool
    exc_info: tuple[type[BaseException], BaseException, TracebackType] | None
    extra: dict[str, Any] | None
    headers: dict[str, Any]
    def __init__(
        self,
        enforce_csrf_checks: bool = ...,
        raise_request_exception: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **defaults: Any,
    ) -> None: ...
    @override
    def request(self, **request: Any) -> _MonkeyPatchedWSGIResponse: ...
    @override
    def get(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: _GetDataType = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedWSGIResponse: ...
    @override
    def post(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: Any = ...,
        content_type: str = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedWSGIResponse: ...
    @override
    def head(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: Any = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedWSGIResponse: ...
    @override
    def options(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: dict[str, str] | str = ...,
        content_type: str = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedWSGIResponse: ...
    @override
    def put(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: Any = ...,
        content_type: str = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedWSGIResponse: ...
    @override
    def patch(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: Any = ...,
        content_type: str = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedWSGIResponse: ...
    @override
    def delete(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: Any = ...,
        content_type: str = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedWSGIResponse: ...
    @override
    def trace(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: Any = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedWSGIResponse: ...

class AsyncClient(ClientMixin, _AsyncRequestFactory[Awaitable[_MonkeyPatchedASGIResponse]]):
    handler: AsyncClientHandler
    raise_request_exception: bool
    exc_info: Any
    extra: dict[str, Any] | None
    headers: dict[str, Any]
    def __init__(
        self,
        enforce_csrf_checks: bool = ...,
        raise_request_exception: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **defaults: Any,
    ) -> None: ...
    @override
    async def request(self, **request: Any) -> _MonkeyPatchedASGIResponse: ...
    @override
    async def get(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: _GetDataType = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedASGIResponse: ...
    @override
    async def post(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: Any = ...,
        content_type: str = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedASGIResponse: ...
    @override
    async def head(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: Any = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedASGIResponse: ...
    @override
    async def options(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: dict[str, str] | str = ...,
        content_type: str = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedASGIResponse: ...
    @override
    async def put(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: Any = ...,
        content_type: str = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedASGIResponse: ...
    @override
    async def patch(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: Any = ...,
        content_type: str = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedASGIResponse: ...
    @override
    async def delete(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: Any = ...,
        content_type: str = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedASGIResponse: ...
    @override
    async def trace(  # type: ignore[override]
        self,
        path: _StrOrPromise,
        data: Any = ...,
        follow: bool = ...,
        secure: bool = ...,
        *,
        headers: Mapping[str, Any] | None = ...,
        query_params: Mapping[Any, Any] | None = ...,
        **extra: Any,
    ) -> _MonkeyPatchedASGIResponse: ...

__all__ = (
    "AsyncClient",
    "AsyncRequestFactory",
    "Client",
    "RedirectCycleError",
    "RequestFactory",
    "encode_file",
    "encode_multipart",
)



from typing import Any, NamedTuple

from django.db.backends.base.introspection import BaseDatabaseIntrospection
from django.db.backends.mysql.base import DatabaseWrapper

FieldInfo: Any

class InfoLine(NamedTuple):
    col_name: str
    data_type: str
    max_len: int | None
    num_prec: int | None
    num_scale: int | None
    extra: str
    column_default: str | None
    collation: str | None
    is_unsigned: bool
    comment: str

class TableInfo(NamedTuple):
    name: str
    type: str
    comment: str

class DatabaseIntrospection(BaseDatabaseIntrospection):
    connection: DatabaseWrapper
    data_types_reverse: Any
    def get_storage_engine(self, cursor: Any, table_name: str) -> str: ...



from typing import Any

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models.expressions import Combinable, Func
from django.db.models.functions.mixins import FixDecimalInputMixin, NumericOutputFieldMixin
from django.db.models.lookups import Transform
from django.db.models.sql.compiler import SQLCompiler, _AsSqlType

class Abs(Transform): ...
class ACos(NumericOutputFieldMixin, Transform): ...
class ASin(NumericOutputFieldMixin, Transform): ...
class ATan(NumericOutputFieldMixin, Transform): ...
class ATan2(NumericOutputFieldMixin, Func): ...

class Ceil(Transform):
    def as_oracle(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...

class Cos(NumericOutputFieldMixin, Transform): ...

class Cot(NumericOutputFieldMixin, Transform):
    def as_oracle(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...

class Degrees(NumericOutputFieldMixin, Transform):
    def as_oracle(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...

class Exp(NumericOutputFieldMixin, Transform): ...
class Floor(Transform): ...
class Ln(NumericOutputFieldMixin, Transform): ...
class Log(FixDecimalInputMixin, NumericOutputFieldMixin, Func): ...
class Mod(FixDecimalInputMixin, NumericOutputFieldMixin, Func): ...

class Pi(NumericOutputFieldMixin, Func):
    def as_oracle(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...

class Power(NumericOutputFieldMixin, Func): ...

class Radians(NumericOutputFieldMixin, Transform):
    def as_oracle(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...

class Random(NumericOutputFieldMixin, Func):
    def as_mysql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...
    def as_oracle(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...

class Round(FixDecimalInputMixin, Transform):
    def __init__(self, expression: Combinable | str, precision: int = 0, **extra: Any) -> None: ...

class Sin(NumericOutputFieldMixin, Transform): ...
class Sqrt(NumericOutputFieldMixin, Transform): ...
class Tan(NumericOutputFieldMixin, Transform): ...
class Sign(Transform): ...



from collections.abc import Sequence

from django.core.mail.backends.base import BaseEmailBackend
from django.core.mail.message import EmailMessage
from typing_extensions import override

class EmailBackend(BaseEmailBackend):
    @override
    def send_messages(self, messages: Sequence[EmailMessage]) -> int: ...



from collections.abc import Callable, Mapping, Sequence
from typing import Any, Literal, TypeAlias

from django.http.request import QueryDict
from django.http.response import HttpResponseBase
from django.urls.resolvers import ResolverMatch
from django.utils.functional import _StrPromise

# https://github.com/python/typeshed/blob/87f599dc8312ac67b941b5f2b47274534a1a2d3a/stdlib/urllib/parse.pyi#L136-L138
_QueryType: TypeAlias = (
    Mapping[Any, Any] | Mapping[Any, Sequence[Any]] | Sequence[tuple[Any, Any]] | Sequence[tuple[Any, Sequence[Any]]]
)

def resolve(path: str | _StrPromise, urlconf: str | None = None) -> ResolverMatch: ...

# NOTE: make sure `reverse` and `reverse_lazy` objects are in sync:
def reverse(
    viewname: Callable[..., HttpResponseBase] | str | None,
    urlconf: str | None = None,
    args: Sequence[Any] | None = None,
    kwargs: dict[str, Any] | None = None,
    current_app: str | None = None,
    *,
    query: QueryDict | _QueryType | None = None,
    fragment: str | None = None,
) -> str: ...
def reverse_lazy(
    viewname: Callable[..., HttpResponseBase] | str | None,
    urlconf: str | None = None,
    args: Sequence[Any] | None = None,
    kwargs: dict[str, Any] | None = None,
    current_app: str | None = None,
    *,
    query: QueryDict | _QueryType | None = None,
    fragment: str | None = None,
) -> _StrPromise: ...
def clear_url_caches() -> None: ...
def set_script_prefix(prefix: str) -> None: ...
def get_script_prefix() -> str: ...
def clear_script_prefix() -> None: ...
def set_urlconf(urlconf_name: str | None) -> None: ...
def get_urlconf(default: str | None = None) -> str | None: ...
def is_valid_path(path: str, urlconf: str | None = None) -> Literal[False] | ResolverMatch: ...
def translate_url(url: str, lang_code: str) -> str: ...



import datetime
from collections.abc import Callable, Iterator, Mapping, Sequence
from decimal import Decimal
from enum import Enum
from typing import Any, ClassVar, Generic, Literal, TypeAlias, TypeVar

from django.core.exceptions import FieldError
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models import Q, fields
from django.db.models.fields import Field
from django.db.models.lookups import Lookup, Transform
from django.db.models.query import QuerySet
from django.db.models.sql.compiler import SQLCompiler, _AsSqlType, _ParamsT
from django.db.models.sql.query import Query
from django.utils.deconstruct import _Deconstructible
from django.utils.functional import cached_property
from typing_extensions import Never, Self, override

_Numeric: TypeAlias = float | Decimal
_ExprListCompatible: TypeAlias = Sequence[BaseExpression | F | str] | BaseExpression | F | str

class SQLiteNumericMixin:
    def as_sqlite(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...

class OutputFieldIsNoneError(FieldError): ...

class Combinable:
    ADD: str
    SUB: str
    MUL: str
    DIV: str
    POW: str
    MOD: str
    BITAND: str
    BITOR: str
    BITLEFTSHIFT: str
    BITRIGHTSHIFT: str
    BITXOR: str
    def __neg__(self) -> CombinedExpression: ...
    def __add__(self, other: datetime.timedelta | Combinable | _Numeric | str | None) -> CombinedExpression: ...
    def __sub__(self, other: datetime.timedelta | Combinable | _Numeric) -> CombinedExpression: ...
    def __mul__(self, other: datetime.timedelta | Combinable | _Numeric) -> CombinedExpression: ...
    def __truediv__(self, other: Combinable | _Numeric) -> CombinedExpression: ...
    def __mod__(self, other: int | Combinable) -> CombinedExpression: ...
    def __pow__(self, other: _Numeric | Combinable) -> CombinedExpression: ...
    def __and__(self, other: Combinable | Q) -> Q: ...
    def bitand(self, other: int) -> CombinedExpression: ...
    def bitleftshift(self, other: int) -> CombinedExpression: ...
    def bitrightshift(self, other: int) -> CombinedExpression: ...
    def __xor__(self, other: Combinable | Q) -> Q: ...
    def bitxor(self, other: int) -> CombinedExpression: ...
    def __or__(self, other: Combinable | Q) -> Q: ...
    def bitor(self, other: int) -> CombinedExpression: ...
    def __radd__(self, other: datetime.datetime | _Numeric | Combinable | None) -> CombinedExpression: ...
    def __rsub__(self, other: _Numeric | Combinable) -> CombinedExpression: ...
    def __rmul__(self, other: _Numeric | Combinable) -> CombinedExpression: ...
    def __rtruediv__(self, other: _Numeric | Combinable) -> CombinedExpression: ...
    def __rmod__(self, other: int | Combinable) -> CombinedExpression: ...
    def __rpow__(self, other: _Numeric | Combinable) -> CombinedExpression: ...
    def __rand__(self, other: Any) -> Combinable: ...
    def __ror__(self, other: Any) -> Combinable: ...
    def __rxor__(self, other: Any) -> Combinable: ...
    def __invert__(self) -> NegatedExpression[Combinable]: ...

class BaseExpression:
    empty_result_set_value: Any
    is_summary: bool
    filterable: bool
    window_compatible: bool
    allowed_default: bool
    constraint_validation_compatible: bool
    set_returning: bool
    allows_composite_expressions: bool
    def __init__(self, output_field: Field | None = None) -> None: ...
    def get_db_converters(self, connection: BaseDatabaseWrapper) -> list[Callable]: ...
    def get_source_expressions(self) -> list[Any]: ...
    def set_source_expressions(self, exprs: Sequence[Combinable | Expression]) -> None: ...
    def as_sql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
    @cached_property
    def contains_aggregate(self) -> bool: ...
    @cached_property
    def contains_over_clause(self) -> bool: ...
    @cached_property
    def contains_column_references(self) -> bool: ...
    @cached_property
    def contains_subquery(self) -> bool: ...
    def resolve_expression(
        self,
        query: Any | None = None,
        allow_joins: bool = True,
        reuse: set[str] | None = None,
        summarize: bool = False,
        for_save: bool = False,
    ) -> Self: ...
    @property
    def conditional(self) -> bool: ...
    @property
    def field(self) -> Field: ...
    @cached_property
    def output_field(self) -> Field: ...
    @cached_property
    def convert_value(self) -> Callable: ...
    def get_lookup(self, lookup: str) -> type[Lookup] | None: ...
    def get_transform(self, name: str) -> type[Transform] | None: ...
    def relabeled_clone(self, change_map: Mapping[str, str]) -> Self: ...
    def replace_expressions(self, replacements: Mapping[Self, Any]) -> Self: ...
    def get_refs(self) -> set[str]: ...
    def copy(self) -> Self: ...
    def prefix_references(self, prefix: str) -> Self: ...
    def get_group_by_cols(self) -> list[BaseExpression]: ...
    def get_source_fields(self) -> list[Field | None]: ...
    def asc(self, **kwargs: Any) -> OrderBy: ...
    def desc(self, **kwargs: Any) -> OrderBy: ...
    def reverse_ordering(self) -> BaseExpression: ...
    def flatten(self) -> Iterator[BaseExpression]: ...
    def select_format(self, compiler: SQLCompiler, sql: str, params: _ParamsT) -> _AsSqlType: ...
    def get_expression_for_validation(self) -> BaseExpression: ...

class Expression(_Deconstructible, BaseExpression, Combinable):
    @cached_property
    def identity(self) -> tuple[Any, ...]: ...

def register_combinable_fields(
    lhs: type[Field],
    connector: str,
    rhs: type[Field],
    result: type[Field],
) -> None: ...

class CombinedExpression(SQLiteNumericMixin, Expression):
    @cached_property
    @override
    def allowed_default(self) -> bool: ...  # type: ignore[override]
    connector: str
    lhs: Combinable
    rhs: Combinable
    def __init__(self, lhs: Combinable, connector: str, rhs: Combinable, output_field: Field | None = None) -> None: ...

class DurationExpression(CombinedExpression):
    def compile(self, side: Combinable, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...

class TemporalSubtraction(CombinedExpression):
    output_field: ClassVar[fields.DurationField]
    def __init__(self, lhs: Combinable, rhs: Combinable) -> None: ...

class F(_Deconstructible, Combinable):
    allowed_default: ClassVar[bool]
    def __init__(self, name: str) -> None: ...
    def __getitem__(self, subscript: int | slice) -> Sliced: ...
    def __contains__(self, other: Any) -> Never: ...
    def resolve_expression(
        self,
        query: Any | None = None,
        allow_joins: bool = True,
        reuse: set[str] | None = None,
        summarize: bool = False,
        for_save: bool = False,
    ) -> Expression: ...
    def replace_expressions(self, replacements: Mapping[F, Any]) -> F: ...
    def asc(self, **kwargs: Any) -> OrderBy: ...
    def desc(self, **kwargs: Any) -> OrderBy: ...
    def copy(self) -> Self: ...

class Sliced(F):
    def __init__(self, obj: F, subscript: int | slice) -> None: ...
    @override
    def resolve_expression(
        self,
        query: Any | None = None,
        allow_joins: bool = True,
        reuse: set[str] | None = None,
        summarize: bool = False,
        for_save: bool = False,
    ) -> Func: ...

class ResolvedOuterRef(F):
    contains_aggregate: ClassVar[bool]
    contains_over_clause: ClassVar[bool]
    def as_sql(self, *args: Any, **kwargs: Any) -> _AsSqlType: ...
    def relabeled_clone(self, relabels: Mapping[str, str]) -> Self: ...
    def get_group_by_cols(self) -> list[BaseExpression]: ...

class OuterRef(F):
    contains_aggregate: ClassVar[bool]
    contains_over_clause: ClassVar[bool]
    def __init__(self, name: str | OuterRef) -> None: ...
    def relabeled_clone(self, relabels: Any) -> Self: ...

class Func(SQLiteNumericMixin, Expression):
    @cached_property
    @override
    def allowed_default(self) -> bool: ...  # type: ignore[override]
    function: str | None = None
    template: str
    arg_joiner: str
    arity: int | None
    source_expressions: list[Expression]
    extra: dict[Any, Any]
    def __init__(self, *expressions: Any, output_field: Field | None = None, **extra: Any) -> None: ...
    @override
    def as_sql(
        self,
        compiler: SQLCompiler,
        connection: BaseDatabaseWrapper,
        function: str | None = None,
        template: str | None = None,
        arg_joiner: str | None = None,
        **extra_context: Any,
    ) -> _AsSqlType: ...

class Value(Expression):
    value: Any
    for_save: bool
    def __init__(self, value: Any, output_field: Field | None = None) -> None: ...
    @property
    @override
    def empty_result_set_value(self) -> Any: ...

class RawSQL(Expression):
    params: list[Any]
    sql: str
    def __init__(self, sql: str, params: Sequence[Any], output_field: Field | None = None) -> None: ...

class Star(Expression): ...

class DatabaseDefault(Expression):
    def __init__(self, expression: Expression, output_field: Field | None = None) -> None: ...

class Col(Expression):
    target: Field
    alias: str
    contains_column_references: Literal[True]
    possibly_multivalued: Literal[False]
    def __init__(self, alias: str, target: Field, output_field: Field | None = None) -> None: ...
    @override
    def relabeled_clone(self, relabels: Mapping[str, str]) -> Self: ...

class ColPairs(Expression):
    alias: str
    targets: Sequence[Field]
    sources: Sequence[Field]
    def __init__(
        self, alias: str, targets: Sequence[Field], sources: Sequence[Field], output_field: Field | None
    ) -> None: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[Col]: ...
    def get_cols(self) -> list[Col]: ...
    @override
    def relabeled_clone(self, relabels: Mapping[str, str]) -> Self: ...

class Ref(Expression):
    def __init__(self, refs: str, source: Expression) -> None: ...
    @override
    def relabeled_clone(self, relabels: Mapping[str, str]) -> Self: ...

class ExpressionList(Func):
    def __init__(
        self, *expressions: BaseExpression | Combinable, output_field: Field | None = None, **extra: Any
    ) -> None: ...

class OrderByList(ExpressionList):
    @classmethod
    def from_param(cls, context: str, param: _ExprListCompatible | None) -> Self | None: ...

_E = TypeVar("_E", bound=Q | Combinable)

class ExpressionWrapper(Expression, Generic[_E]):
    @property
    @override
    def allowed_default(self) -> bool: ...  # type: ignore[override]
    def __init__(self, expression: _E, output_field: Field) -> None: ...
    expression: _E

class NegatedExpression(ExpressionWrapper[_E]):
    def __init__(self, expression: _E) -> None: ...
    @override
    def __invert__(self) -> _E: ...  # type: ignore[override]

class When(Expression):
    @cached_property
    @override
    def allowed_default(self) -> bool: ...  # type: ignore[override]
    template: str
    condition: Any
    result: Any
    def __init__(self, condition: Any | None = None, then: Any | None = None, **lookups: Any) -> None: ...
    @override
    def as_sql(
        self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, template: str | None = None, **extra_context: Any
    ) -> _AsSqlType: ...

class Case(Expression):
    @cached_property
    @override
    def allowed_default(self) -> bool: ...  # type: ignore[override]
    template: str
    case_joiner: str
    cases: Any
    default: Any
    extra: Any
    def __init__(
        self, *cases: Any, default: Any | None = None, output_field: Field | None = None, **extra: Any
    ) -> None: ...
    @override
    def as_sql(
        self,
        compiler: SQLCompiler,
        connection: BaseDatabaseWrapper,
        template: str | None = None,
        case_joiner: str | None = None,
        **extra_context: Any,
    ) -> _AsSqlType: ...

class Subquery(BaseExpression, Combinable):
    template: str
    subquery: bool
    query: Query
    extra: dict[Any, Any]
    def __init__(
        self, queryset: Query | QuerySet | Subquery, output_field: Field | None = None, **extra: Any
    ) -> None: ...
    @property
    def external_aliases(self) -> set[str]: ...
    def get_external_cols(self) -> list[Col]: ...
    @override
    def as_sql(
        self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, template: str | None = None, **extra_context: Any
    ) -> _AsSqlType: ...

class Exists(Subquery):
    output_field: ClassVar[fields.BooleanField]
    def __init__(self, queryset: Query | QuerySet | Subquery, **kwargs: Any) -> None: ...

class OrderBy(Expression):
    template: str
    nulls_first: bool
    nulls_last: bool
    descending: bool
    expression: Expression | F | Subquery
    allows_composite_expressions: bool
    def __init__(
        self,
        expression: Expression | F | Subquery,
        descending: bool = False,
        nulls_first: bool | None = None,
        nulls_last: bool | None = None,
    ) -> None: ...
    @override
    def as_sql(
        self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, template: str | None = None, **extra_context: Any
    ) -> _AsSqlType: ...
    def as_oracle(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
    @override
    def asc(self) -> None: ...  # type: ignore[override]
    @override
    def desc(self) -> None: ...  # type: ignore[override]

class Window(SQLiteNumericMixin, Expression):
    template: str
    contains_aggregate: bool
    contains_over_clause: bool
    partition_by: ExpressionList | None
    order_by: OrderByList | None
    def __init__(
        self,
        expression: BaseExpression,
        partition_by: _ExprListCompatible | None = None,
        order_by: _ExprListCompatible | None = None,
        frame: WindowFrame | None = None,
        output_field: Field | None = None,
    ) -> None: ...
    @override
    def as_sql(
        self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, template: str | None = None
    ) -> _AsSqlType: ...
    @override
    def as_sqlite(  # type: ignore[override]
        self, compiler: SQLCompiler, connection: BaseDatabaseWrapper
    ) -> _AsSqlType: ...

class WindowFrameExclusion(Enum):
    CURRENT_ROW = "CURRENT ROW"
    GROUP = "GROUP"
    TIES = "TIES"
    NO_OTHERS = "NO OTHERS"

class WindowFrame(Expression):
    template: str
    def __init__(
        self,
        start: int | None = None,
        end: int | None = None,
        exclusion: WindowFrameExclusion | None = None,
    ) -> None: ...
    def get_exclusion(self) -> str: ...
    def window_frame_start_end(
        self, connection: BaseDatabaseWrapper, start: int | None, end: int | None
    ) -> tuple[int, int]: ...

class RowRange(WindowFrame):
    frame_type: str

class ValueRange(WindowFrame):
    frame_type: str



from collections.abc import Iterator
from typing import Any, Literal, NamedTuple

from django.db.migrations.state import ModelState, ProjectState
from django.db.models import Field, Model

COMPILED_REGEX_TYPE: Any
RECURSIVE_RELATIONSHIP_CONSTANT: str

class RegexObject:
    pattern: str
    flags: int
    def __init__(self, obj: Any) -> None: ...

def get_migration_name_timestamp() -> str: ...
def resolve_relation(
    model: str | type[Model], app_label: str | None = None, model_name: str | None = None
) -> tuple[str, str]: ...

class FieldReference(NamedTuple):
    to: Any
    through: Any

def field_references(
    model_tuple: tuple[str, str],
    field: Field,
    reference_model_tuple: tuple[str, str],
    reference_field_name: str | None = None,
    reference_field: Field | None = None,
) -> Literal[False] | FieldReference: ...
def get_references(
    state: ProjectState,
    model_tuple: tuple[str, str],
    field_tuple: tuple[()] | tuple[str, Field] = (),
) -> Iterator[tuple[ModelState, str, Field, FieldReference]]: ...
def field_is_referenced(state: ProjectState, model_tuple: tuple[str, str], field_tuple: tuple[str, Field]) -> bool: ...



from typing import Any, ClassVar

from django import forms
from django.contrib.flatpages.models import FlatPage

class FlatpageForm(forms.ModelForm[FlatPage]):
    url: Any

    class Meta:
        model: ClassVar[type[FlatPage]]
        fields: ClassVar[str]

    def clean_url(self) -> str: ...



import smtplib
import ssl
import threading
from typing import Any

from _typeshed import StrOrBytesPath
from django.core.mail.backends.base import BaseEmailBackend
from django.utils.functional import cached_property

class EmailBackend(BaseEmailBackend):
    host: str
    port: int
    username: str
    password: str
    use_tls: bool
    use_ssl: bool
    timeout: int | None
    ssl_keyfile: StrOrBytesPath | None
    ssl_certfile: StrOrBytesPath | None
    connection: smtplib.SMTP_SSL | smtplib.SMTP | None
    _lock: threading.RLock
    def __init__(
        self,
        host: str | None = None,
        port: int | None = None,
        username: str | None = None,
        password: str | None = None,
        use_tls: bool | None = None,
        fail_silently: bool = False,
        use_ssl: bool | None = None,
        timeout: int | None = None,
        ssl_keyfile: StrOrBytesPath | None = None,
        ssl_certfile: StrOrBytesPath | None = None,
        **kwargs: Any,
    ) -> None: ...
    @property
    def connection_class(self) -> type[smtplib.SMTP_SSL | smtplib.SMTP]: ...
    @cached_property
    def ssl_context(self) -> ssl.SSLContext: ...
    def prep_address(self, address: str, force_ascii: bool = True) -> str: ...



from typing import Any, Protocol, TypeVar, type_check_only

_RT_co = TypeVar("_RT_co", covariant=True)

@type_check_only
class _SupportsReplace(Protocol[_RT_co]):
    def __replace__(self, /, *_: Any, **changes: Any) -> _RT_co: ...

def replace(obj: _SupportsReplace[_RT_co], /, **changes: Any) -> _RT_co: ...



import datetime
from collections.abc import AsyncIterator, Collection, Iterable, Iterator, Mapping, Sequence
from typing import Any, Generic, Literal, NoReturn, TypeVar, overload

from django.core.checks.messages import CheckMessage
from django.db.models.base import Model
from django.db.models.expressions import Combinable, OrderBy
from django.db.models.query import Prefetch, QuerySet, RawQuerySet, _LookupT, _PrefetchedQuerySetT, _ToAttrT
from typing_extensions import Self

_T = TypeVar("_T", bound=Model, covariant=True)

class BaseManager(Generic[_T]):
    creation_counter: int
    auto_created: bool
    use_in_migrations: bool
    name: str
    model: type[_T]
    _db: str | None
    def __new__(cls, *args: Any, **kwargs: Any) -> Self: ...
    def __init__(self) -> None: ...
    def __class_getitem__(cls, *args: Any, **kwargs: Any) -> type[Self]: ...
    def deconstruct(
        self,
    ) -> tuple[bool, str | None, str | None, Sequence[Any] | None, dict[str, Any] | None]: ...
    def check(self, **kwargs: Any) -> list[CheckMessage]: ...
    @classmethod
    def from_queryset(cls, queryset_class: type[QuerySet[_T]], class_name: str | None = None) -> type[Self]: ...
    @classmethod
    def _get_queryset_methods(cls, queryset_class: type) -> dict[str, Any]: ...
    def contribute_to_class(self, cls: type[Model], name: str) -> None: ...
    def db_manager(self, using: str | None = None, hints: dict[str, Model] | None = None) -> Self: ...
    @property
    def db(self) -> str: ...
    def get_queryset(self) -> QuerySet[_T]: ...
    def all(self) -> QuerySet[_T]: ...

class Manager(BaseManager[_T]):
    # NOTE: The following methods are in common with QuerySet, but note that the use of QuerySet as a return type
    # rather than a self-type (_QS), since Manager's QuerySet-like methods return QuerySets and not Managers.
    def iterator(self, chunk_size: int | None = ...) -> Iterator[_T]: ...
    def aiterator(self, chunk_size: int = ...) -> AsyncIterator[_T]: ...
    def aggregate(self, *args: Any, **kwargs: Any) -> dict[str, Any]: ...
    async def aaggregate(self, *args: Any, **kwargs: Any) -> dict[str, Any]: ...
    def get(self, *args: Any, **kwargs: Any) -> _T: ...
    async def aget(self, *args: Any, **kwargs: Any) -> _T: ...
    def create(self, **kwargs: Any) -> _T: ...
    async def acreate(self, **kwargs: Any) -> _T: ...
    def bulk_create(
        self,
        objs: Iterable[_T],
        batch_size: int | None = ...,
        ignore_conflicts: bool = ...,
        update_conflicts: bool = ...,
        update_fields: Collection[str] | None = ...,
        unique_fields: Collection[str] | None = ...,
    ) -> list[_T]: ...
    async def abulk_create(
        self,
        objs: Iterable[_T],
        batch_size: int | None = ...,
        ignore_conflicts: bool = ...,
        update_conflicts: bool = ...,
        update_fields: Collection[str] | None = ...,
        unique_fields: Collection[str] | None = ...,
    ) -> list[_T]: ...
    def bulk_update(self, objs: Iterable[_T], fields: Iterable[str], batch_size: int | None = ...) -> int: ...
    async def abulk_update(self, objs: Iterable[_T], fields: Iterable[str], batch_size: int | None = ...) -> int: ...
    def get_or_create(self, defaults: Mapping[str, Any] | None = ..., **kwargs: Any) -> tuple[_T, bool]: ...
    async def aget_or_create(self, defaults: Mapping[str, Any] | None = ..., **kwargs: Any) -> tuple[_T, bool]: ...
    def update_or_create(
        self,
        defaults: Mapping[str, Any] | None = ...,
        create_defaults: Mapping[str, Any] | None = ...,
        **kwargs: Any,
    ) -> tuple[_T, bool]: ...
    async def aupdate_or_create(
        self,
        defaults: Mapping[str, Any] | None = ...,
        create_defaults: Mapping[str, Any] | None = ...,
        **kwargs: Any,
    ) -> tuple[_T, bool]: ...
    def earliest(self, *fields: str | OrderBy) -> _T: ...
    async def aearliest(self, *fields: str | OrderBy) -> _T: ...
    def latest(self, *fields: str | OrderBy) -> _T: ...
    async def alatest(self, *fields: str | OrderBy) -> _T: ...
    def first(self) -> _T | None: ...
    async def afirst(self) -> _T | None: ...
    def last(self) -> _T | None: ...
    async def alast(self) -> _T | None: ...
    def in_bulk(self, id_list: Iterable[Any] | None = ..., *, field_name: str = ...) -> dict[Any, _T]: ...
    async def ain_bulk(self, id_list: Iterable[Any] | None = ..., *, field_name: str = ...) -> dict[Any, _T]: ...
    def update(self, **kwargs: Any) -> int: ...
    async def aupdate(self, **kwargs: Any) -> int: ...
    def exists(self) -> bool: ...
    async def aexists(self) -> bool: ...
    def explain(self, *, format: str | None = ..., **options: Any) -> str: ...
    async def aexplain(self, *, format: str | None = ..., **options: Any) -> str: ...
    def contains(self, obj: Model) -> bool: ...
    async def acontains(self, obj: Model) -> bool: ...
    def raw(
        self,
        raw_query: str,
        params: Any = ...,
        translations: dict[str, str] | None = ...,
        using: str | None = ...,
    ) -> RawQuerySet[_T]: ...
    # The type of values may be overridden to be more specific in the mypy plugin, depending on the fields param
    def values(self, *fields: str | Combinable, **expressions: Any) -> QuerySet[_T, dict[str, Any]]: ...
    # The type of values_list may be overridden to be more specific in the mypy plugin, depending on the fields param
    def values_list(self, *fields: str | Combinable, flat: bool = ..., named: bool = ...) -> QuerySet[_T, Any]: ...
    def dates(
        self,
        field_name: str,
        kind: Literal["year", "month", "week", "day"],
        order: Literal["ASC", "DESC"] = "ASC",
    ) -> QuerySet[_T, datetime.date]: ...
    def datetimes(
        self,
        field_name: str,
        kind: Literal["year", "month", "week", "day", "hour", "minute", "second"],
        order: Literal["ASC", "DESC"] = "ASC",
        tzinfo: datetime.tzinfo | None = None,
    ) -> QuerySet[_T, datetime.datetime]: ...
    def none(self) -> QuerySet[_T]: ...
    def filter(self, *args: Any, **kwargs: Any) -> QuerySet[_T]: ...
    def exclude(self, *args: Any, **kwargs: Any) -> QuerySet[_T]: ...
    def complex_filter(self, filter_obj: Any) -> QuerySet[_T]: ...
    def count(self) -> int: ...
    async def acount(self) -> int: ...
    def union(self, *other_qs: QuerySet[Model, Any], all: bool = ...) -> QuerySet[_T]: ...
    def intersection(self, *other_qs: QuerySet[Model, Any]) -> QuerySet[_T]: ...
    def difference(self, *other_qs: QuerySet[Model, Any]) -> QuerySet[_T]: ...
    def select_for_update(
        self, nowait: bool = ..., skip_locked: bool = ..., of: Sequence[str] = ..., no_key: bool = ...
    ) -> QuerySet[_T]: ...
    @overload
    def select_related(self, clear: None, /) -> QuerySet[_T]: ...
    @overload
    def select_related(self, *fields: str) -> QuerySet[_T]: ...
    @overload
    def prefetch_related(self, clear: None, /) -> QuerySet[_T]: ...
    @overload
    def prefetch_related(self, *lookups: str | Prefetch[_LookupT, _PrefetchedQuerySetT, _ToAttrT]) -> QuerySet[_T]: ...
    def annotate(self, *args: Any, **kwargs: Any) -> QuerySet[_T]: ...
    def alias(self, *args: Any, **kwargs: Any) -> QuerySet[_T]: ...
    def order_by(self, *field_names: str | Combinable) -> QuerySet[_T]: ...
    def distinct(self, *field_names: str) -> QuerySet[_T]: ...
    # extra() return type won't be supported any time soon
    def extra(
        self,
        select: dict[str, Any] | None = ...,
        where: list[str] | None = ...,
        params: list[Any] | None = ...,
        tables: list[str] | None = ...,
        order_by: Sequence[str] | None = ...,
        select_params: Sequence[Any] | None = ...,
    ) -> QuerySet[Any]: ...
    def reverse(self) -> QuerySet[_T]: ...
    @overload
    def defer(self, clear: None, /) -> QuerySet[_T]: ...
    @overload
    def defer(self, *fields: str) -> QuerySet[_T]: ...
    def only(self, *fields: str) -> QuerySet[_T]: ...
    def using(self, alias: str | None) -> QuerySet[_T]: ...

class ManagerDescriptor:
    manager: BaseManager
    def __init__(self, manager: BaseManager) -> None: ...
    @overload
    def __get__(self, instance: None, cls: type[Model] | None = None) -> BaseManager: ...
    @overload
    def __get__(self, instance: Model, cls: type[Model] | None = None) -> NoReturn: ...

class EmptyManager(Manager[_T]):
    def __init__(self, model: type[_T]) -> None: ...



from collections.abc import Awaitable, Callable, Iterator, Mapping, Sequence
from logging import Logger
from typing import IO, Any, TypeAlias, TypeVar

from django.core.files import uploadedfile
from django.core.handlers import base
from django.http.request import HttpRequest, _ImmutableQueryDict
from django.http.response import HttpResponseBase
from django.urls.resolvers import ResolverMatch, URLResolver
from django.utils.datastructures import MultiValueDict
from django.utils.functional import cached_property
from typing_extensions import override

_ReceiveCallback: TypeAlias = Callable[[], Awaitable[Mapping[str, Any]]]

_SendCallback: TypeAlias = Callable[[Mapping[str, Any]], Awaitable[None]]

logger: Logger

def get_script_prefix(scope: Mapping[str, Any]) -> str: ...

class ASGIRequest(HttpRequest):
    body_receive_timeout: int
    scope: Mapping[str, Any]
    resolver_match: ResolverMatch | None
    script_name: str | None
    path_info: str
    path: str
    method: str
    META: dict[str, Any]
    def __init__(self, scope: Mapping[str, Any], body_file: IO[bytes]) -> None: ...
    @cached_property
    @override
    def GET(self) -> _ImmutableQueryDict: ...  # type: ignore[override]
    POST: _ImmutableQueryDict
    @property
    @override
    def FILES(self) -> MultiValueDict[str, uploadedfile.UploadedFile]: ...  # type: ignore[override]
    @cached_property
    @override
    def COOKIES(self) -> dict[str, str]: ...  # type: ignore[override]

_T = TypeVar("_T")

class ASGIHandler(base.BaseHandler):
    request_class: type[ASGIRequest]
    chunk_size: int
    def __init__(self) -> None: ...
    async def __call__(
        self,
        scope: dict[str, Any],
        receive: _ReceiveCallback,
        send: _SendCallback,
    ) -> None: ...
    async def handle(
        self, scope: dict[str, Any], receive: _ReceiveCallback, send: _SendCallback
    ) -> HttpResponseBase | None: ...
    async def listen_for_disconnect(self, receive: _ReceiveCallback) -> None: ...
    async def run_get_response(self, request: HttpRequest) -> HttpResponseBase: ...
    async def read_body(self, receive: _ReceiveCallback) -> IO[bytes]: ...
    def create_request(
        self, scope: Mapping[str, Any], body_file: IO[bytes]
    ) -> tuple[ASGIRequest, None] | tuple[None, HttpResponseBase]: ...
    def handle_uncaught_exception(
        self, request: HttpRequest, resolver: URLResolver, exc_info: Any
    ) -> HttpResponseBase: ...
    async def send_response(self, response: HttpResponseBase, send: _SendCallback) -> None: ...
    @classmethod
    def chunk_bytes(cls, data: Sequence[_T]) -> Iterator[tuple[Sequence[_T], bool]]: ...



from abc import ABCMeta, abstractmethod
from typing import Any

from django.tasks.base import Task, TaskResult

class BaseTaskBackend(metaclass=ABCMeta):
    task_class: type[Task]
    supports_defer: bool
    supports_async_task: bool
    supports_get_result: bool
    supports_priority: bool
    alias: str
    queues: list[str]
    options: dict[str, Any]
    def __init__(self, alias: str, params: dict[str, Any]) -> None: ...
    def validate_task(self, task: Task) -> None: ...
    @abstractmethod
    def enqueue(self, task: Task, args: list[Any], kwargs: dict[str, Any]) -> TaskResult: ...
    async def aenqueue(self, task: Task, args: list[Any], kwargs: dict[str, Any]) -> TaskResult: ...
    def get_result(self, result_id: str) -> TaskResult: ...
    async def aget_result(self, result_id: str) -> TaskResult: ...
    def check(self, **kwargs: Any) -> list[Any]: ...



from typing import Any

from django import forms
from django.contrib import admin
from django.contrib.auth.models import Group, _UserType
from django.db.models.fields.related import ManyToManyField
from django.forms.models import ModelMultipleChoiceField
from django.http.request import HttpRequest
from django.http.response import HttpResponse
from typing_extensions import override

class GroupAdmin(admin.ModelAdmin[Group]):
    @override
    def formfield_for_manytomany(
        self, db_field: ManyToManyField, request: HttpRequest | None = ..., **kwargs: Any
    ) -> ModelMultipleChoiceField | None: ...

class UserAdmin(admin.ModelAdmin[_UserType]):
    change_user_password_template: Any
    add_fieldsets: Any
    add_form: Any
    change_password_form: Any
    @override
    def get_form(  # type: ignore[override]
        self, request: HttpRequest, obj: _UserType | None = ..., **kwargs: Any
    ) -> type[forms.ModelForm[_UserType]]: ...
    @override
    def lookup_allowed(self, lookup: str, value: str, request: HttpRequest) -> bool: ...
    def user_change_password(self, request: HttpRequest, id: str, form_url: str = ...) -> HttpResponse: ...



from typing import Any

from django.db.backends.sqlite3.base import DatabaseWrapper as SQLiteDatabaseWrapper
from typing_extensions import override

from .client import SpatiaLiteClient
from .features import DatabaseFeatures
from .introspection import SpatiaLiteIntrospection
from .operations import SpatiaLiteOperations
from .schema import SpatialiteSchemaEditor

class DatabaseWrapper(SQLiteDatabaseWrapper):
    SchemaEditorClass: type[SpatialiteSchemaEditor]
    client_class: type[SpatiaLiteClient]
    features_class: type[DatabaseFeatures]
    introspection_class: type[SpatiaLiteIntrospection]
    ops_class: type[SpatiaLiteOperations]
    lib_spatialite_paths: str
    ops: SpatiaLiteOperations
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @override
    def get_new_connection(self, conn_params: Any) -> Any: ...
    @override
    def prepare_database(self) -> None: ...



from collections.abc import AsyncIterator, Iterable, Iterator, Sequence, Sized
from typing import ClassVar, Generic, Protocol, TypeVar, overload, type_check_only

from django.db.models.fields import _ErrorMessagesDict
from django.utils.functional import _StrPromise, cached_property
from typing_extensions import override

class UnorderedObjectListWarning(RuntimeWarning): ...
class InvalidPage(Exception): ...
class PageNotAnInteger(InvalidPage): ...
class EmptyPage(InvalidPage): ...

_T = TypeVar("_T")

@type_check_only
class _SupportsPagination(Protocol[_T], Sized, Iterable):
    @overload
    def __getitem__(self, index: int, /) -> _T: ...
    @overload
    def __getitem__(self, index: slice, /) -> _SupportsPagination[_T]: ...

class BasePaginator(Generic[_T]):
    ELLIPSIS: ClassVar[_StrPromise]
    default_error_messages: ClassVar[_ErrorMessagesDict]
    error_messages: _ErrorMessagesDict
    object_list: _SupportsPagination[_T]
    per_page: int
    orphans: int
    allow_empty_first_page: bool
    def __init__(
        self,
        object_list: _SupportsPagination[_T],
        per_page: int | str,
        orphans: int = 0,
        allow_empty_first_page: bool = True,
        error_messages: _ErrorMessagesDict | None = None,
    ) -> None: ...

class Paginator(BasePaginator[_T]):
    def __iter__(self) -> Iterator[Page[_T]]: ...
    def validate_number(self, number: int | float | str) -> int: ...
    def get_page(self, number: int | float | str | None) -> Page[_T]: ...
    def page(self, number: int | str) -> Page[_T]: ...
    @cached_property
    def count(self) -> int: ...
    @cached_property
    def num_pages(self) -> int: ...
    @property
    def page_range(self) -> range: ...
    def get_elided_page_range(
        self, number: int | float | str = 1, *, on_each_side: int = 3, on_ends: int = 2
    ) -> Iterator[str | int]: ...

class AsyncPaginator(BasePaginator[_T]):
    def __aiter__(self) -> AsyncIterator[AsyncPage[_T]]: ...
    async def avalidate_number(self, number: int | float | str) -> int: ...
    async def aget_page(self, number: int | float | str | None) -> AsyncPage[_T]: ...
    async def apage(self, number: int | str) -> AsyncPage[_T]: ...
    async def acount(self) -> int: ...
    async def anum_pages(self) -> int: ...
    async def apage_range(self) -> range: ...
    async def aget_elided_page_range(
        self, number: int | float | str = 1, *, on_each_side: int = 3, on_ends: int = 2
    ) -> AsyncIterator[str | int]: ...

class Page(Sequence[_T]):
    object_list: _SupportsPagination[_T]
    number: int
    paginator: Paginator[_T]
    def __init__(
        self,
        object_list: _SupportsPagination[_T],
        number: int,
        paginator: Paginator[_T],
    ) -> None: ...
    @overload
    @override
    def __getitem__(self, index: int) -> _T: ...
    @overload
    @override
    def __getitem__(self, index: slice) -> Sequence[_T]: ...  # Avoid override error
    @override
    def __len__(self) -> int: ...
    def has_next(self) -> bool: ...
    def has_previous(self) -> bool: ...
    def has_other_pages(self) -> bool: ...
    def next_page_number(self) -> int: ...
    def previous_page_number(self) -> int: ...
    def start_index(self) -> int: ...
    def end_index(self) -> int: ...

class AsyncPage(Generic[_T]):
    object_list: _SupportsPagination[_T]
    number: int
    paginator: AsyncPaginator[_T]
    def __init__(
        self,
        object_list: _SupportsPagination[_T],
        number: int,
        paginator: AsyncPaginator[_T],
    ) -> None: ...
    def __aiter__(self) -> AsyncIterator[_T]: ...
    def __len__(self) -> int: ...
    def __reversed__(self) -> Iterator[_T]: ...
    @overload
    def __getitem__(self, index: int) -> _T: ...
    @overload
    def __getitem__(self, index: slice) -> list[_T]: ...
    async def aget_object_list(self) -> list[_T]: ...
    async def ahas_next(self) -> bool: ...
    async def ahas_previous(self) -> bool: ...
    async def ahas_other_pages(self) -> bool: ...
    async def anext_page_number(self) -> int: ...
    async def aprevious_page_number(self) -> int: ...
    async def astart_index(self) -> int: ...
    async def aend_index(self) -> int: ...



import json
from collections.abc import Callable
from typing import Any, ClassVar, TypeVar

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models import Model, lookups
from django.db.models.expressions import Expression
from django.db.models.fields import TextField
from django.db.models.lookups import FieldGetDbPrepValueMixin, PostgresOperatorLookup, Transform
from django.db.models.sql.compiler import SQLCompiler, _AsSqlType
from django.utils.functional import _StrOrPromise
from typing_extensions import Self, override

from . import Field
from .mixins import CheckFieldDefaultMixin

# __set__ value type
_ST = TypeVar("_ST", contravariant=True, default=Any)
# __get__ return type
_GT = TypeVar("_GT", covariant=True, default=Any)

class JSONField(CheckFieldDefaultMixin, Field[_ST, _GT]):
    encoder: type[json.JSONEncoder] | None
    decoder: type[json.JSONDecoder] | None
    def __init__(
        self,
        verbose_name: _StrOrPromise | None = None,
        name: str | None = None,
        encoder: type[json.JSONEncoder] | None = None,
        decoder: type[json.JSONDecoder] | None = None,
        **kwargs: Any,
    ) -> None: ...
    def from_db_value(self, value: str | None, expression: Expression, connection: BaseDatabaseWrapper) -> Any: ...
    @override
    def get_transform(self, name: str) -> type[Transform] | KeyTransformFactory: ...  # type: ignore[override]
    @override
    def value_to_string(self, obj: Model) -> Any: ...
    @override
    def formfield(self, **kwargs: Any) -> Any: ...  # type: ignore[override]

class DataContains(FieldGetDbPrepValueMixin, PostgresOperatorLookup): ...
class ContainedBy(FieldGetDbPrepValueMixin, PostgresOperatorLookup): ...

class HasKeyLookup(PostgresOperatorLookup):
    logical_operator: str | None
    def compile_json_path_final_key(self, connection: BaseDatabaseWrapper, key_transform: Any) -> str: ...
    @override
    def as_sql(
        self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, template: str | None = None
    ) -> _AsSqlType: ...
    def as_mysql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
    def as_sqlite(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...

class HasKey(HasKeyLookup):
    postgres_operator: str

class HasKeys(HasKeyLookup):
    postgres_operator: str
    logical_operator: str

class HasAnyKeys(HasKeys):
    postgres_operator: str
    logical_operator: str

class HasKeyOrArrayIndex(HasKey):
    @override
    def compile_json_path_final_key(self, connection: BaseDatabaseWrapper, key_transform: Any) -> str: ...

class JSONExact(lookups.Exact): ...

class CaseInsensitiveMixin:
    def process_rhs(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...

class JSONIContains(CaseInsensitiveMixin, lookups.IContains): ...

class KeyTransform(Transform):
    key_name: str
    postgres_operator: str
    postgres_nested_operator: str
    def __init__(self, key_name: Any, *args: Any, **kwargs: Any) -> None: ...
    def preprocess_lhs(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> Any: ...
    def as_mysql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
    def as_oracle(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
    def as_postgresql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
    @override
    def as_sqlite(  # type: ignore[override]
        self, compiler: SQLCompiler, connection: BaseDatabaseWrapper
    ) -> _AsSqlType: ...

class KeyTextTransform(KeyTransform):
    postgres_operator: str
    postgres_nested_operator: str
    output_field: ClassVar[TextField]
    @override
    def as_mysql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...
    @classmethod
    def from_lookup(cls, lookup: str) -> Self: ...

KT: Callable[[str], KeyTextTransform]

class KeyTransformTextLookupMixin:
    def __init__(self, key_transform: Any, *args: Any, **kwargs: Any) -> None: ...

class KeyTransformIsNull(lookups.IsNull):
    def as_sqlite(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...

class KeyTransformIn(lookups.In): ...
class KeyTransformExact(JSONExact): ...
class KeyTransformIExact(CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IExact): ...
class KeyTransformIContains(CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IContains): ...
class KeyTransformStartsWith(KeyTransformTextLookupMixin, lookups.StartsWith): ...
class KeyTransformIStartsWith(CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IStartsWith): ...
class KeyTransformEndsWith(KeyTransformTextLookupMixin, lookups.EndsWith): ...
class KeyTransformIEndsWith(CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IEndsWith): ...
class KeyTransformRegex(KeyTransformTextLookupMixin, lookups.Regex): ...
class KeyTransformIRegex(CaseInsensitiveMixin, KeyTransformTextLookupMixin, lookups.IRegex): ...

class KeyTransformNumericLookupMixin:
    def process_rhs(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper) -> _AsSqlType: ...

class KeyTransformLt(KeyTransformNumericLookupMixin, lookups.LessThan): ...
class KeyTransformLte(KeyTransformNumericLookupMixin, lookups.LessThanOrEqual): ...
class KeyTransformGt(KeyTransformNumericLookupMixin, lookups.GreaterThan): ...
class KeyTransformGte(KeyTransformNumericLookupMixin, lookups.GreaterThanOrEqual): ...

class KeyTransformFactory:
    key_name: Any
    def __init__(self, key_name: Any) -> None: ...
    def __call__(self, *args: Any, **kwargs: Any) -> KeyTransform: ...

__all__ = ["JSONField"]



from collections.abc import Callable
from typing import Any

from django.contrib.contenttypes.models import ContentType
from django.core.checks.messages import CheckMessage
from django.db.models.base import Model
from django.db.models.expressions import Combinable
from django.db.models.fields import Field, _AllLimitChoicesTo
from django.db.models.fields.mixins import FieldCacheMixin
from django.db.models.fields.related import ForeignObject
from django.db.models.fields.related_descriptors import ReverseManyToOneDescriptor
from django.db.models.fields.reverse_related import ForeignObjectRel
from django.db.models.query import QuerySet
from django.db.models.query_utils import FilteredRelation, PathInfo
from django.db.models.sql.where import WhereNode
from django.utils.functional import cached_property
from typing_extensions import override

class GenericForeignKey(FieldCacheMixin, Field):
    # django-stubs implementation only fields
    _pyi_private_set_type: Any | Combinable
    _pyi_private_get_type: Any
    # attributes
    hidden: bool
    is_relation: bool
    many_to_many: bool
    many_to_one: bool
    one_to_many: bool
    one_to_one: bool
    related_model: Any
    ct_field: str
    fk_field: str
    for_concrete_model: bool
    def __init__(
        self, ct_field: str = "content_type", fk_field: str = "object_id", for_concrete_model: bool = True
    ) -> None: ...
    name: Any
    model: Any
    @override
    def contribute_to_class(self, cls: type[Model], name: str, **kwargs: Any) -> None: ...  # type: ignore[override]
    @override
    def get_attname_column(self) -> tuple[str, None]: ...  # type: ignore[override]
    @cached_property
    def ct_field_attname(self) -> str: ...
    @override
    def get_filter_kwargs_for_object(self, obj: Model) -> dict[str, ContentType | None]: ...
    def get_forward_related_filter(self, obj: Model) -> dict[str, int]: ...
    @override
    def check(self, **kwargs: Any) -> list[CheckMessage]: ...
    def get_content_type(
        self,
        obj: Model | None = None,
        id: int | None = None,
        using: str | None = None,
        model: type[Model] | None = None,
    ) -> ContentType: ...
    def get_prefetch_querysets(
        self, instances: list[Model] | QuerySet, querysets: list[QuerySet] | None = None
    ) -> tuple[list[Model], Callable[..., Any], Callable[..., Any], bool, str, bool]: ...
    @override
    def __get__(self, instance: Model | None, cls: type[Model] | None = ...) -> Any | None: ...  # type: ignore[override]
    @override
    def __set__(self, instance: Model, value: Any | None) -> None: ...

class GenericRel(ForeignObjectRel):
    field: GenericRelation
    def __init__(
        self,
        field: GenericRelation,
        to: type[Model] | str,
        related_name: str | None = None,
        related_query_name: str | None = None,
        limit_choices_to: _AllLimitChoicesTo | None = None,
    ) -> None: ...

class GenericRelation(ForeignObject[Any, Any]):
    rel_class: type[GenericRel]
    mti_inherited: bool
    object_id_field_name: str
    content_type_field_name: str
    for_concrete_model: bool
    def __init__(
        self,
        to: type[Model] | str,
        object_id_field: str = "object_id",
        content_type_field: str = "content_type",
        for_concrete_model: bool = True,
        related_query_name: str | None = None,
        limit_choices_to: _AllLimitChoicesTo | None = None,
        **kwargs: Any,
    ) -> None: ...
    @override
    def resolve_related_fields(self) -> list[tuple[Field, Field]]: ...
    @override
    def get_local_related_value(self, instance: Model) -> tuple[Any, ...]: ...
    @override
    def get_foreign_related_value(self, instance: Model) -> tuple[Any, ...]: ...
    @override
    def get_path_info(self, filtered_relation: FilteredRelation | None = None) -> list[PathInfo]: ...
    @override
    def get_reverse_path_info(self, filtered_relation: FilteredRelation | None = None) -> list[PathInfo]: ...
    @override
    def contribute_to_class(self, cls: type[Model], name: str, **kwargs: Any) -> None: ...  # type: ignore[override]
    def get_content_type(self) -> ContentType: ...
    @override
    def get_extra_restriction(self, alias: str | None, remote_alias: str) -> WhereNode: ...
    def bulk_related_objects(self, objs: list[Model], using: str = "default") -> QuerySet: ...

class ReverseGenericManyToOneDescriptor(ReverseManyToOneDescriptor): ...

def create_generic_related_manager(superclass: Any, rel: Any) -> type[Any]: ...  # GenericRelatedObjectManager



from typing import Any, Generic, TypeVar

from django.db.models import Model, Prefetch
from django.db.models.query import QuerySet
from typing_extensions import override

# The type of the lookup passed to Prefetch(...)
# This will be specialized to a `LiteralString` in the plugin for further processing and validation
_LookupT = TypeVar("_LookupT", bound=str, covariant=True)
# The type of the querysets passed to GenericPrefetch(...)
_PrefetchedQuerySetsT = TypeVar(
    "_PrefetchedQuerySetsT", bound=list[QuerySet[Model]], covariant=True, default=list[QuerySet[Model]]
)
# The attribute name to store the prefetched list[_PrefetchedQuerySetT]
# This will be specialized to a `LiteralString` in the plugin for further processing and validation
_ToAttrT = TypeVar("_ToAttrT", bound=str, covariant=True, default=str)

class GenericPrefetch(Prefetch, Generic[_LookupT, _PrefetchedQuerySetsT, _ToAttrT]):
    def __init__(self, lookup: _LookupT, querysets: _PrefetchedQuerySetsT, to_attr: _ToAttrT | None = None) -> None: ...
    @override
    def __getstate__(self) -> dict[str, Any]: ...
    @override
    def get_current_querysets(self, level: int) -> _PrefetchedQuerySetsT | None: ...



import os
import sys
from collections import defaultdict
from collections.abc import Iterable, Iterator, Mapping, Sequence
from contextlib import contextmanager
from functools import cached_property
from typing import TYPE_CHECKING, Any, Literal, Union

from django.core.exceptions import FieldDoesNotExist, FieldError
from django.db import models
from django.db.models.base import Model
from django.db.models.constants import LOOKUP_SEP
from django.db.models.expressions import Expression
from django.db.models.fields import AutoField, CharField, Field
from django.db.models.fields.related import ForeignKey, RelatedField
from django.db.models.fields.reverse_related import ForeignObjectRel
from django.db.models.lookups import Exact
from django.db.models.sql.query import Query
from mypy.checker import TypeChecker
from mypy.nodes import TypeInfo
from mypy.plugin import MethodContext
from mypy.typeanal import make_optional_type
from mypy.types import AnyType, Instance, ProperType, TypeOfAny, UnionType, get_proper_type
from mypy.types import Type as MypyType

from mypy_django_plugin.exceptions import UnregisteredModelError
from mypy_django_plugin.lib import fullnames, helpers

# This import fails when `psycopg2` is not installed, avoid crashing the plugin.
try:
    from django.contrib.postgres.fields import ArrayField
except ImportError:

    class ArrayField:  # type: ignore[no-redef]
        pass


if TYPE_CHECKING:
    from django.apps.registry import Apps
    from django.conf import LazySettings
    from django.db.models.options import _AnyField


@contextmanager
def temp_environ() -> Iterator[None]:
    """Allow the ability to set os.environ temporarily"""
    environ = dict(os.environ)
    try:
        yield
    finally:
        os.environ.clear()
        os.environ.update(environ)


def initialize_django(settings_module: str) -> tuple["Apps", "LazySettings"]:
    with temp_environ():
        os.environ["DJANGO_SETTINGS_MODULE"] = settings_module

        # add current directory to sys.path
        sys.path.append(os.getcwd())

        from django.apps import apps
        from django.conf import settings

        apps.get_swappable_settings_name.cache_clear()  # type: ignore[attr-defined]
        apps.clear_cache()

        if not settings.configured:
            settings._setup()  # type: ignore[misc]
        apps.populate(settings.INSTALLED_APPS)

    assert apps.apps_ready, "Apps are not ready"
    assert settings.configured, "Settings are not configured"

    return apps, settings


class LookupsAreUnsupported(Exception):
    pass


def get_field_type_from_model_type_info(info: TypeInfo | None, field_name: str) -> Instance | None:
    if info is None:
        return None
    field_node = info.get(field_name)
    if field_node is None:
        return None
    field_type = get_proper_type(field_node.type)
    if not isinstance(field_type, Instance):
        return None
    # Field declares a set and a get type arg. Fallback to `None` when we can't find any args
    if len(field_type.args) != 2:
        return None
    return field_type


def _get_field_set_type_from_model_type_info(info: TypeInfo | None, field_name: str) -> MypyType | None:
    field_type = get_field_type_from_model_type_info(info, field_name)
    if field_type is not None:
        return field_type.args[0]
    return None


def _get_field_get_type_from_model_type_info(info: TypeInfo | None, field_name: str) -> MypyType | None:
    field_type = get_field_type_from_model_type_info(info, field_name)
    if field_type is not None:
        return field_type.args[1]
    return None


class DjangoContext:
    def __init__(self, django_settings_module: str) -> None:
        self.django_settings_module = django_settings_module

        apps, settings = initialize_django(self.django_settings_module)
        self.apps_registry = apps
        self.settings = settings

    @cached_property
    def model_modules(self) -> dict[str, dict[str, type[Model]]]:
        """All modules that contain Django models."""
        modules: dict[str, dict[str, type[Model]]] = defaultdict(dict)
        for concrete_model_cls in self.apps_registry.get_models(include_auto_created=True, include_swapped=True):
            modules[concrete_model_cls.__module__][concrete_model_cls.__name__] = concrete_model_cls
            # collect abstract=True models
            for model_cls in concrete_model_cls.mro()[1:]:
                if issubclass(model_cls, Model) and hasattr(model_cls, "_meta") and model_cls._meta.abstract:
                    modules[model_cls.__module__][model_cls.__name__] = model_cls
        return modules

    def get_model_class_by_fullname(self, fullname: str) -> type[Model] | None:
        """Returns None if Model is abstract"""
        module, _, model_cls_name = fullname.rpartition(".")
        return self.model_modules.get(module, {}).get(model_cls_name)

    def get_model_fields(self, model_cls: type[Model]) -> Iterator["Field[Any, Any]"]:
        for field in model_cls._meta.get_fields():
            if isinstance(field, Field):
                yield field

    def get_model_foreign_keys(self, model_cls: type[Model]) -> Iterator["ForeignKey[Any, Any]"]:
        for field in model_cls._meta.get_fields():
            if isinstance(field, ForeignKey):
                yield field

    def get_model_related_fields(self, model_cls: type[Model]) -> Iterator["RelatedField[Any, Any]"]:
        """Get model forward relations"""
        for field in model_cls._meta.get_fields():
            if isinstance(field, RelatedField):
                yield field

    def get_model_relations(self, model_cls: type[Model]) -> Iterator[ForeignObjectRel]:
        """Get model reverse relations"""
        for field in model_cls._meta.get_fields():
            if isinstance(field, ForeignObjectRel):
                yield field

    def get_field_lookup_exact_type(
        self, api: TypeChecker, field: Union["Field[Any, Any]", ForeignObjectRel]
    ) -> MypyType:
        if isinstance(field, RelatedField | ForeignObjectRel):
            related_model_cls = self.get_field_related_model_cls(field)
            rel_model_info = helpers.lookup_class_typeinfo(api, related_model_cls)
            if rel_model_info is None:
                return AnyType(TypeOfAny.explicit)

            primary_key_field = self.get_primary_key_field(related_model_cls)
            primary_key_type = self.get_field_get_type(api, rel_model_info, primary_key_field, method="init")

            model_and_primary_key_type = UnionType.make_union([Instance(rel_model_info, []), primary_key_type])
            return make_optional_type(model_and_primary_key_type)

        field_info = helpers.lookup_class_typeinfo(api, field.__class__)
        if field_info is None:
            return AnyType(TypeOfAny.explicit)
        return helpers.get_private_descriptor_type(field_info, "_pyi_lookup_exact_type", is_nullable=field.null)

    def get_related_target_field(
        self, related_model_cls: type[Model], field: "ForeignKey[Any, Any]"
    ) -> "Field[Any, Any] | None":
        # ForeignKey only supports one `to_fields` item (ForeignObject supports many)
        assert len(field.to_fields) == 1
        to_field_name = field.to_fields[0]
        if to_field_name:
            rel_field = related_model_cls._meta.get_field(to_field_name)
            if not isinstance(rel_field, Field):
                return None  # Not supported
            return rel_field
        return self.get_primary_key_field(related_model_cls)

    def get_primary_key_field(self, model_cls: type[Model]) -> "Field[Any, Any]":
        for field in model_cls._meta.get_fields():
            if isinstance(field, Field):
                if field.primary_key:
                    return field
        raise ValueError("No primary key defined")

    def get_expected_types(self, api: TypeChecker, model_cls: type[Model], *, method: str) -> dict[str, MypyType]:
        contenttypes_in_apps = self.apps_registry.is_installed("django.contrib.contenttypes")

        expected_types = {}
        # add pk if not abstract=True
        if not model_cls._meta.abstract:
            primary_key_field = self.get_primary_key_field(model_cls)
            field_set_type = self.get_field_set_type(api, primary_key_field, method=method)
            expected_types["pk"] = field_set_type

        model_info = helpers.lookup_class_typeinfo(api, model_cls)
        for field in model_cls._meta.get_fields():
            if contenttypes_in_apps:
                from django.contrib.contenttypes.fields import GenericForeignKey

                if isinstance(field, GenericForeignKey):
                    # it's generic, so cannot set specific model
                    field_name = field.name
                    gfk_info = helpers.lookup_class_typeinfo(api, field.__class__)
                    if gfk_info is None:
                        gfk_set_type: MypyType = AnyType(TypeOfAny.unannotated)
                    else:
                        gfk_set_type = helpers.get_private_descriptor_type(
                            gfk_info, "_pyi_private_set_type", is_nullable=True
                        )
                    expected_types[field_name] = gfk_set_type
                    continue

            if isinstance(field, Field):
                field_name = field.attname
                # Can not determine target_field for recursive relationship when model is abstract
                if field.related_model == "self" and model_cls._meta.abstract:
                    continue
                # Try to retrieve set type from a model's TypeInfo object and fallback to retrieving it manually
                # from django-stubs own declaration. This is to align with the setter types declared for
                # assignment.
                field_set_type = _get_field_set_type_from_model_type_info(
                    model_info, field_name
                ) or self.get_field_set_type(api, field, method=method)
                expected_types[field_name] = field_set_type

                if isinstance(field, ForeignKey):
                    field_name = field.name
                    foreign_key_info = helpers.lookup_class_typeinfo(api, field.__class__)
                    if foreign_key_info is None:
                        # maybe there's no type annotation for the field
                        expected_types[field_name] = AnyType(TypeOfAny.unannotated)
                        continue

                    try:
                        related_model = self.get_field_related_model_cls(field)
                    except UnregisteredModelError:
                        # Recognise the field but don't say anything about its type..
                        expected_types[field_name] = AnyType(TypeOfAny.from_error)
                        continue

                    if related_model._meta.proxy_for_model is not None:
                        related_model = related_model._meta.proxy_for_model

                    related_model_info = helpers.lookup_class_typeinfo(api, related_model)
                    if related_model_info is None:
                        expected_types[field_name] = AnyType(TypeOfAny.unannotated)
                        continue

                    is_nullable = self.get_field_nullability(field, method)
                    foreign_key_set_type = helpers.get_private_descriptor_type(
                        foreign_key_info, "_pyi_private_set_type", is_nullable=is_nullable
                    )
                    model_set_type = helpers.convert_any_to_type(foreign_key_set_type, Instance(related_model_info, []))

                    expected_types[field_name] = model_set_type

        return expected_types

    @cached_property
    def all_registered_model_classes(self) -> set[type[models.Model]]:
        model_classes = self.apps_registry.get_models()

        all_model_bases = set()
        for model_cls in model_classes:
            for base_cls in model_cls.mro():
                if issubclass(base_cls, models.Model):
                    all_model_bases.add(base_cls)

        return all_model_bases

    @cached_property
    def model_class_fullnames_by_label(self) -> Mapping[str, str]:
        return {
            klass._meta.label: helpers.get_class_fullname(klass)
            for klass in self.all_registered_model_classes
            if klass is not models.Model
        }

    def get_field_nullability(self, field: Union["Field[Any, Any]", ForeignObjectRel], method: str | None) -> bool:
        if method in ("values", "values_list"):
            return field.null

        nullable = field.null
        if not nullable and isinstance(field, CharField) and field.blank:
            return True
        if method == "__init__":
            if (isinstance(field, Field) and field.primary_key) or isinstance(field, ForeignKey):
                return True
        if method == "create":
            if isinstance(field, AutoField):
                return True
        if isinstance(field, Field) and field.has_default():
            return True
        return nullable

    def get_field_set_type(
        self, api: TypeChecker, field: Union["Field[Any, Any]", ForeignObjectRel], *, method: str
    ) -> MypyType:
        """Get a type of __set__ for this specific Django field."""
        target_field = field
        if isinstance(field, ForeignKey):
            try:
                # We gotta be careful for exceptions when we're triggering '__get__'.
                # Related model could very well be unresolvable
                target_field = field.target_field
            except ValueError:
                return AnyType(TypeOfAny.from_error)

        field_info = helpers.lookup_class_typeinfo(api, target_field.__class__)
        if field_info is None:
            return AnyType(TypeOfAny.from_error)

        field_set_type = helpers.get_private_descriptor_type(
            field_info, "_pyi_private_set_type", is_nullable=self.get_field_nullability(field, method)
        )
        if isinstance(target_field, ArrayField):
            argument_field_type = self.get_field_set_type(api, target_field.base_field, method=method)
            field_set_type = helpers.convert_any_to_type(field_set_type, argument_field_type)
        return field_set_type

    def get_field_get_type(
        self,
        api: TypeChecker,
        model_info: TypeInfo | None,
        field: Union["Field[Any, Any]", ForeignObjectRel],
        *,
        method: str,
    ) -> MypyType:
        """Get a type of __get__ for this specific Django field."""
        if isinstance(field, Field):
            get_type = _get_field_get_type_from_model_type_info(model_info, field.attname)
            if get_type is not None:
                return get_type

        field_info = helpers.lookup_class_typeinfo(api, field.__class__)
        if field_info is None:
            return AnyType(TypeOfAny.unannotated)

        is_nullable = self.get_field_nullability(field, method)
        if isinstance(field, RelatedField):
            related_model_cls = self.get_field_related_model_cls(field)
            rel_model_info = helpers.lookup_class_typeinfo(api, related_model_cls)

            if method in ("values", "values_list"):
                primary_key_field = self.get_primary_key_field(related_model_cls)
                return self.get_field_get_type(api, rel_model_info, primary_key_field, method=method)

            model_info = helpers.lookup_class_typeinfo(api, related_model_cls)
            if model_info is None:
                return AnyType(TypeOfAny.unannotated)

            return Instance(model_info, [])
        return helpers.get_private_descriptor_type(field_info, "_pyi_private_get_type", is_nullable=is_nullable)

    def get_field_related_model_cls(self, field: Union["RelatedField[Any, Any]", ForeignObjectRel]) -> type[Model]:
        if isinstance(field, RelatedField):
            related_model_cls = field.remote_field.model
        else:
            related_model_cls = field.field.model

        if related_model_cls is None:
            raise UnregisteredModelError

        if isinstance(related_model_cls, str):
            if related_model_cls == "self":  # type: ignore[unreachable]
                # same model
                related_model_cls = field.model
            elif "." not in related_model_cls:
                # same file model
                related_model_fullname = f"{field.model.__module__}.{related_model_cls}"
                related_model_cls = self.get_model_class_by_fullname(related_model_fullname)
                if related_model_cls is None:
                    raise UnregisteredModelError
            else:
                try:
                    related_model_cls = self.apps_registry.get_model(related_model_cls)
                except LookupError as e:
                    raise UnregisteredModelError from e

        return related_model_cls

    def _resolve_field_from_parts(
        self, field_parts: Iterable[str], model_cls: type[Model]
    ) -> tuple[Union["Field[Any, Any]", ForeignObjectRel], type[Model]]:
        currently_observed_model = model_cls
        field: _AnyField | None = None
        for field_part in field_parts:
            if field_part == "pk":
                field = self.get_primary_key_field(currently_observed_model)
                continue

            field = currently_observed_model._meta.get_field(field_part)
            if isinstance(field, RelatedField):
                currently_observed_model = self.get_field_related_model_cls(field)
                model_name = currently_observed_model._meta.model_name
                if model_name is not None and field_part == (model_name + "_id"):
                    field = self.get_primary_key_field(currently_observed_model)

            if isinstance(field, ForeignObjectRel):
                currently_observed_model = self.get_field_related_model_cls(field)

        # Guaranteed by `query.solve_lookup_type` before.
        assert isinstance(field, Field | ForeignObjectRel)
        return field, currently_observed_model

    def solve_lookup_type(
        self, model_cls: type[Model], lookup: str
    ) -> tuple[Sequence[str], Sequence[str], Expression | Literal[False]] | None:
        query = Query(model_cls)
        if (lookup == "pk" or lookup.startswith("pk__")) and query.get_meta().pk is None:
            # Primary key lookup when no primary key field is found, model is presumably
            # abstract and we can't say anything about 'pk'.
            return None
        try:
            return query.solve_lookup_type(lookup)
        # This occurs when the following conditions are met:
        # - model_cls._meta.abstract = True
        # - part of the lookup is a foreign key defined on model_cls where the 'to' argument is a string
        # On abstract models the 'to' parameter is only coalesced into the actual class whenever a subclass of it is
        # instantiated, therefore it is never swapped out for abstract base classes.
        except AttributeError:
            pass
        query_parts = lookup.split(LOOKUP_SEP)
        try:
            field = query.get_meta().get_field(query_parts[0])
        except FieldDoesNotExist:
            return None

        if len(query_parts) == 1:
            return [], [query_parts[0]], False

        if not isinstance(field, RelatedField | ForeignObjectRel):
            return None

        related_model = self.get_field_related_model_cls(field)
        sub_query = Query(related_model).solve_lookup_type(LOOKUP_SEP.join(query_parts[1:]))
        entire_query_parts = [query_parts[0], *sub_query[1]]
        return sub_query[0], entire_query_parts, sub_query[2]

    def resolve_lookup_into_field(
        self, model_cls: type[Model], lookup: str
    ) -> tuple[Union["Field[Any, Any]", ForeignObjectRel, None], type[Model]]:
        solved_lookup = self.solve_lookup_type(model_cls, lookup)
        if solved_lookup is None:
            return None, model_cls
        lookup_parts, field_parts, _ = solved_lookup
        if lookup_parts:
            raise LookupsAreUnsupported()
        return self._resolve_field_from_parts(field_parts, model_cls)

    def _resolve_lookup_type_from_lookup_class(
        self, ctx: MethodContext, lookup_cls: type, field: Union["Field[Any, Any]", ForeignObjectRel] | None = None
    ) -> MypyType | None:
        """Resolve the expected type for a lookup class (used both for regular fields and annotated fields)

        Args:
            ctx
            lookup_cls: The Django lookup class (e.g., IsNull, Contains)
            field: Optional field for resolving Field-dependent types (None for annotated fields)

        Returns:
            The resolved type, or None if it couldn't be determined
        """
        lookup_info = helpers.lookup_class_typeinfo(helpers.get_typechecker_api(ctx), lookup_cls)
        if lookup_info is None:
            return None

        for lookup_base in helpers.iter_bases(lookup_info):
            if lookup_base.args and isinstance((lookup_type := get_proper_type(lookup_base.args[0])), Instance):
                # if it's Field, consider lookup_type a __get__ of current field
                if lookup_type.type.fullname == fullnames.FIELD_FULLNAME:
                    if field is None:
                        # No field available (e.g., annotation), can't resolve further
                        return None
                    field_info = helpers.lookup_class_typeinfo(helpers.get_typechecker_api(ctx), field.__class__)
                    if field_info is None:
                        return None
                    return get_proper_type(
                        helpers.get_private_descriptor_type(field_info, "_pyi_private_get_type", is_nullable=field.null)
                    )
                return lookup_type

        return None

    def _resolve_annotated_field_lookup(
        self, ctx: MethodContext, lookup: str, model_instance: Instance
    ) -> MypyType | None:
        """Resolve the expected type for a lookup on an annotated field.

        Args:
            ctx
            lookup: The full lookup string (e.g., 'total__gte' or 'total').
            model_instance: The model instance containing extra_attrs from annotations.

        Returns:
            The resolved mypy type for the lookup, or None if the base field
            is not found in the model's annotations.
        """
        if not helpers.is_annotated_model(model_instance.type) or not model_instance.extra_attrs:
            return None

        lookup_base_field, *annotation_lookup_parts = lookup.split(LOOKUP_SEP)

        if lookup_base_field not in model_instance.extra_attrs.attrs:
            return None

        if annotation_lookup_parts:
            lookup_cls = Field().get_lookup(annotation_lookup_parts[-1])
            if lookup_cls is not None:
                lookup_type = self._resolve_lookup_type_from_lookup_class(ctx, lookup_cls)
                if lookup_type is not None:
                    return lookup_type

        # No lookup suffix or Field-dependent lookup: fall back to annotation type
        return model_instance.extra_attrs.attrs[lookup_base_field]

    def resolve_lookup_expected_type(
        self, ctx: MethodContext, model_cls: type[Model], lookup: str, model_instance: Instance
    ) -> MypyType:
        try:
            # solve_lookup_type uses Django's Query.solve_lookup_type(), which raises
            # FieldError for annotated fields since they don't exist on the actual model...
            solved_lookup = self.solve_lookup_type(model_cls, lookup)
        except FieldError as exc:
            # ...so we handle annotation lookups here
            annotation_lookup = self._resolve_annotated_field_lookup(ctx, lookup, model_instance)
            if annotation_lookup is not None:
                return annotation_lookup

            msg = exc.args[0]
            if model_instance.extra_attrs:
                msg = ", ".join((msg, *model_instance.extra_attrs.attrs.keys()))
            ctx.api.fail(msg, ctx.context)
            return AnyType(TypeOfAny.from_error)

        if solved_lookup is None:
            return AnyType(TypeOfAny.implementation_artifact)
        lookup_parts, field_parts, is_expression = solved_lookup
        if is_expression:
            return AnyType(TypeOfAny.explicit)

        field, _ = self._resolve_field_from_parts(field_parts, model_cls)

        lookup_cls = None
        if lookup_parts:
            lookup = lookup_parts[-1]
            lookup_cls = field.get_lookup(lookup)
            if lookup_cls is None:
                # unknown lookup
                return AnyType(TypeOfAny.explicit)

        if lookup_cls is None or isinstance(lookup_cls, Exact):
            return self.get_field_lookup_exact_type(helpers.get_typechecker_api(ctx), field)

        resolved_type = self._resolve_lookup_type_from_lookup_class(ctx, lookup_cls, field)
        if resolved_type is not None:
            return resolved_type

        return AnyType(TypeOfAny.explicit)

    def resolve_f_expression_type(self, f_expression_type: Instance) -> ProperType:
        return AnyType(TypeOfAny.explicit)

    @cached_property
    def is_contrib_auth_installed(self) -> bool:
        return "django.contrib.auth" in self.settings.INSTALLED_APPS



import re
from typing import Any

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models import Func
from django.db.models.expressions import Combinable
from django.db.models.fields import Field
from django.db.models.sql.compiler import SQLCompiler, _AsSqlType
from typing_extensions import override

class Cast(Func):
    def __init__(self, expression: Combinable | str, output_field: str | Field) -> None: ...
    @override
    def as_sql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...  # type: ignore[override]
    def as_mysql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...
    def as_postgresql(
        self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any
    ) -> _AsSqlType: ...
    def as_oracle(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...

class Coalesce(Func):
    @property
    @override
    def empty_result_set_value(self) -> Any: ...
    def as_oracle(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...

class Collate(Func):
    collation_re: re.Pattern[str]
    def __init__(self, expression: Combinable | str, collation: str) -> None: ...
    @override
    def as_sql(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...  # type: ignore[override]

class Greatest(Func): ...
class Least(Func): ...

class NullIf(Func):
    def as_oracle(self, compiler: SQLCompiler, connection: BaseDatabaseWrapper, **extra_context: Any) -> _AsSqlType: ...



from enum import Enum, auto

from django.db.models.constants import LOOKUP_SEP
from mypy.nodes import MemberExpr, NameExpr, SuperExpr, TypeAlias, TypeInfo, Var
from mypy.plugin import AttributeContext
from mypy.typeanal import make_optional_type
from mypy.types import (
    AnyType,
    Instance,
    LiteralType,
    Overloaded,
    ProperType,
    TupleType,
    TypeOfAny,
    TypeType,
    TypeVarType,
    UnionType,
    get_proper_type,
)
from mypy.types import Type as MypyType

from mypy_django_plugin.lib import fullnames


# TODO: [mypy 1.14+] Remove this backport of `TypeInfo.enum_members`.
def _get_enum_members(info: TypeInfo) -> list[str]:
    try:
        return info.enum_members
    except AttributeError:  # mypy < 1.14
        pass

    return [
        name
        for name, sym in info.names.items()
        if (
            isinstance(sym.node, Var)
            and name not in ("_ignore_", "_order_", "__order__")
            and not name.startswith(LOOKUP_SEP)
            and sym.node.has_explicit_value
        )
    ]


class _LabelLaziness(Enum):
    LAZY = auto()
    NON_LAZY = auto()
    MIXED = auto()

    @classmethod
    def make(cls, lazy: bool | None) -> "_LabelLaziness":
        return cls.LAZY if lazy else cls.NON_LAZY


def _is_lazy_string(ty: MypyType | None) -> bool:
    proper_type = get_proper_type(ty)
    return isinstance(proper_type, Instance) and proper_type.type.has_base(fullnames.STR_PROMISE_FULLNAME)


def _is_lazy_member(node: TypeInfo, member_name: str) -> bool:
    if (sym := node.get(member_name)) is None:
        return False

    _member_type = get_proper_type(sym.type)
    if not isinstance(_member_type, TupleType):
        # Member has auto-generated plain string label - enum.auto() or no explicit label.
        return False

    if _member_type.length() < 2:
        # There need to be at least two items in the tuple.
        return False

    return _is_lazy_string(_member_type.items[-1])


def _label_laziness(node: TypeInfo) -> _LabelLaziness:
    """
    Check whether a choices type has lazy strings for labels.

    This is used to determine whether a choices type uses lazy strings or non-lazy string for
    labels such that a more simple type can be used instead of the default in the stubs.
    """
    assert node.is_enum

    if (sym := node.get("__empty__")) is not None:
        lazy = _is_lazy_string(sym.type)
    else:
        lazy = None

    # TODO: [mypy 1.14+] Use `node.enum_members` and remove `_get_enum_members()` backport.
    for member_name in _get_enum_members(node):
        member_lazy = _is_lazy_member(node, member_name)
        if lazy is None:
            lazy = member_lazy
        elif lazy != member_lazy:
            return _LabelLaziness.MIXED

    return _LabelLaziness.make(lazy)


def _try_replace_label(typ: ProperType, label_laziness: _LabelLaziness) -> MypyType:
    """
    Attempt to replace a label with a modified version.

    If there are no lazy strings for labels, remove the lazy string type, and vice-versa.
    """
    if label_laziness is _LabelLaziness.MIXED:
        return typ

    if not isinstance(typ, UnionType):
        # If it's not a union, then it already is likely just `str` and not lazy.
        return typ

    items = [
        t
        for t in map(get_proper_type, typ.items)
        if isinstance(t, Instance)
        and t.type.has_base(fullnames.STR_PROMISE_FULLNAME) == (label_laziness is _LabelLaziness.LAZY)
    ]

    # If we only have one item use that, otherwise make a new union.
    return UnionType.make_union(items) if len(items) > 1 else items[0]


def _try_replace_value(typ: ProperType, base_type: Instance | None, has_empty_label: bool) -> MypyType:
    """
    Attempt to replace a label with a modified version.

    If the value is of any type, then attempt to use the base type of the choices type.

    If the choices type has `__empty__` defined, then make the value type optional.
    """
    has_unknown_base = isinstance(typ, AnyType)

    if has_empty_label and has_unknown_base and base_type is not None:
        return make_optional_type(base_type)

    if has_empty_label:
        return make_optional_type(typ)

    if has_unknown_base and base_type is not None:
        return base_type

    return typ


def _get_enum_type_from_union_of_literals(typ: UnionType) -> MypyType:
    """
    Attempts to resolve a single enum type from a union of enum literals.

    If this cannot be resolved, the original is returned.
    """
    types = set()

    for item in typ.items:
        item = get_proper_type(item)
        if not isinstance(item, LiteralType) or not item.fallback.type.is_enum:
            # If anything that isn't a literal of an enum type is encountered, return the original.
            return typ
        types.add(item.fallback)

    # If there is only one enum type seen, return that otherwise return the original.
    return types.pop() if len(types) == 1 else typ


def transform_into_proper_attr_type(ctx: AttributeContext) -> MypyType:
    """
    A `get_attribute_hook` to make `.choices` and `.values` optional if `__empty__` is defined.

    When the `__empty__` label is specified, the `choices` and `values` properties can return a
    blank choice/value. This hook will amend the type returned by those properties.
    """
    if isinstance(ctx.context, MemberExpr):
        expr = ctx.context.expr
        name = ctx.context.name
    elif isinstance(ctx.context, SuperExpr):
        expr = ctx.context
        name = ctx.context.name
    else:
        ctx.api.fail("Unable to resolve type of property", ctx.context)
        return AnyType(TypeOfAny.from_error)

    node: TypeInfo | None = None

    if isinstance(expr, MemberExpr | NameExpr):
        if isinstance(expr.node, TypeInfo):
            node = expr.node
        elif isinstance(expr.node, TypeAlias):
            alias = get_proper_type(expr.node.target)
            if isinstance(alias, Instance):
                node = alias.type
    elif isinstance(expr, SuperExpr):
        node = expr.info

    _node_type: ProperType | None = None

    if node is None:
        _node_type = get_proper_type(ctx.api.get_expression_type(expr))
        if isinstance(_node_type, UnionType):
            # If this is a union of enum literals, check if they're all of the same enum type and
            # use that type instead. This situation often occurs where there is comparison to an
            # enum member in a branch.
            _node_type = get_proper_type(_get_enum_type_from_union_of_literals(_node_type))
        if isinstance(_node_type, LiteralType) and _node_type.is_enum_literal():
            _node_type = _node_type.fallback
        if isinstance(_node_type, TypeType):
            _node_type = _node_type.item
        if isinstance(_node_type, TypeVarType):
            _node_type = get_proper_type(_node_type.upper_bound)
        if isinstance(_node_type, Instance):
            node = _node_type.type
        if isinstance(_node_type, Overloaded) and _node_type.is_type_obj():
            node = _node_type.type_object()

    if node is None:
        if isinstance(_node_type, UnionType):
            # Suppress the error for a known case where there are multiple base choices types in a
            # union. In theory this is something that could be handled by extracting all of the
            # base types and making the following code consider all of these types, but that's
            # quite a bit of effort.
            return ctx.default_attr_type
        ctx.api.fail(f"Unable to resolve type of {name} property", ctx.context)
        return AnyType(TypeOfAny.from_error)

    default_attr_type = get_proper_type(ctx.default_attr_type)

    if not node.is_enum or not node.has_base(fullnames.CHOICES_CLASS_FULLNAME):
        return default_attr_type

    # Enums with more than one base will treat the first base as the mixed-in type.
    base_type = node.bases[0] if len(node.bases) > 1 else None

    # When `__empty__` is defined, the `.choices` and `.values` properties will include `None` for
    # the blank choice which is labelled by the value of `__empty__`.
    empty_label = node.get("__empty__")

    # When `__empty__` is not a lazy string and the labels on all members are not lazy strings, the
    # label can be simplified to only be a simple string type. This keeps the benefits of accurate
    # typing when the lazy labels are being used, but reduces the pain of having to manage a union
    # of a simple and lazy string type where it's not necessary.
    label_laziness = _label_laziness(node)

    if (
        name == "choices"
        and isinstance(default_attr_type, Instance)
        and default_attr_type.type.fullname == "builtins.list"
        and len(default_attr_type.args) == 1
    ):
        choice_arg = get_proper_type(default_attr_type.args[0])

        if isinstance(choice_arg, TupleType) and choice_arg.length() == 2:
            value_arg, label_arg = choice_arg.items
            label_arg = get_proper_type(label_arg)
            value_arg = get_proper_type(value_arg)
            new_label_arg = _try_replace_label(label_arg, label_laziness)
            new_value_arg = _try_replace_value(value_arg, base_type, empty_label is not None)
            if new_label_arg is not label_arg or new_value_arg is not value_arg:
                new_choice_arg = choice_arg.copy_modified(items=[new_value_arg, new_label_arg])
                return default_attr_type.copy_modified(args=[new_choice_arg])

    elif (
        name == "labels"
        and isinstance(default_attr_type, Instance)
        and default_attr_type.type.fullname == "builtins.list"
        and len(default_attr_type.args) == 1
    ):
        label_arg = get_proper_type(default_attr_type.args[0])
        new_label_arg = _try_replace_label(label_arg, label_laziness)
        if new_label_arg is not label_arg:
            return default_attr_type.copy_modified(args=[new_label_arg])

    elif (
        name == "values"
        and isinstance(default_attr_type, Instance)
        and default_attr_type.type.fullname == "builtins.list"
        and len(default_attr_type.args) == 1
    ):
        value_arg = get_proper_type(default_attr_type.args[0])
        new_value_arg = _try_replace_value(value_arg, base_type, empty_label is not None)
        if new_value_arg is not value_arg:
            return default_attr_type.copy_modified(args=[new_value_arg])

    elif name == "label":
        return _try_replace_label(default_attr_type, label_laziness)

    elif name == "__empty__" and empty_label is not None:
        return _try_replace_label(default_attr_type, _LabelLaziness.make(_is_lazy_string(empty_label.type)))

    elif name == "value":
        # Pass in `False` because `.value` will never return `None`.
        return _try_replace_value(default_attr_type, base_type, False)

    return default_attr_type



import json
from collections.abc import Iterable, Sequence
from datetime import date, time, timedelta
from datetime import datetime as real_datetime
from decimal import Decimal
from typing import Any

from django.core.management.color import Style
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.backends.utils import CursorWrapper
from django.db.models.base import Model
from django.db.models.constants import OnConflict
from django.db.models.expressions import Case, Col, Expression
from django.db.models.fields import Field
from django.db.models.sql.compiler import SQLCompiler

class BaseDatabaseOperations:
    compiler_module: str
    integer_field_ranges: dict[str, tuple[int, int]]
    set_operators: dict[str, str]
    cast_data_types: dict[Any, Any]
    cast_char_field_without_max_length: Any
    PRECEDING: str
    FOLLOWING: str
    UNBOUNDED_PRECEDING: str
    UNBOUNDED_FOLLOWING: str
    CURRENT_ROW: str
    explain_prefix: str | None
    connection: BaseDatabaseWrapper
    def __init__(self, connection: BaseDatabaseWrapper) -> None: ...
    def __del__(self) -> None: ...
    def autoinc_sql(self, table: str, column: str) -> str | None: ...
    def bulk_batch_size(self, fields: Any, objs: Any) -> int: ...
    def format_for_duration_arithmetic(self, sql: str) -> str: ...
    def cache_key_culling_sql(self) -> str: ...
    def unification_cast_sql(self, output_field: Field) -> str: ...
    def date_extract_sql(self, lookup_type: str, sql: Any, params: Any) -> tuple[str, Any]: ...
    # def date_interval_sql(self, timedelta: None) -> Any: ...
    def date_trunc_sql(self, lookup_type: str, sql: str, params: Any, tzname: str | None = None) -> tuple[str, Any]: ...
    def datetime_cast_date_sql(self, sql: str, params: Any, tzname: str | None) -> tuple[str, Any]: ...
    def datetime_cast_time_sql(self, sql: str, params: Any, tzname: str | None) -> tuple[str, Any]: ...
    def datetime_extract_sql(self, lookup_type: str, sql: str, params: Any, tzname: str | None) -> tuple[str, Any]: ...
    def datetime_trunc_sql(self, lookup_type: str, sql: str, params: Any, tzname: str | None) -> str: ...
    def time_trunc_sql(self, lookup_type: str, sql: str, params: Any, tzname: str | None = None) -> str: ...
    def time_extract_sql(self, lookup_type: str, sql: str, params: Any) -> str: ...
    def deferrable_sql(self) -> str: ...
    def distinct_sql(self, fields: list[str], params: list[Any] | None) -> tuple[list[str], list[str]]: ...
    def force_group_by(self) -> list[str]: ...
    def force_no_ordering(self) -> list[Any]: ...
    def for_update_sql(
        self, nowait: bool = False, skip_locked: bool = False, of: Any = (), no_key: bool = False
    ) -> str: ...
    def limit_offset_sql(self, low_mark: int, high_mark: int | None) -> str: ...
    def bulk_insert_sql(self, fields: Iterable[Field], placeholder_rows: Iterable[str]) -> str: ...
    def last_executed_query(self, cursor: Any, sql: Any, params: Any) -> str: ...
    def last_insert_id(self, cursor: CursorWrapper, table_name: str, pk_name: str) -> int: ...
    def lookup_cast(self, lookup_type: str, internal_type: str | None = None) -> str: ...
    def max_in_list_size(self) -> int | None: ...
    def max_name_length(self) -> int | None: ...
    def no_limit_value(self) -> str | None: ...
    def pk_default_value(self) -> str: ...
    def prepare_sql_script(self, sql: Any) -> list[str]: ...
    def process_clob(self, value: str) -> str: ...
    def returning_columns(self, fields: Iterable[Field]) -> tuple[str, tuple[()]]: ...
    def fetch_returned_rows(self, cursor: CursorWrapper, returning_params: tuple[()]) -> list[tuple[Any, ...]]: ...
    def compiler(self, compiler_name: str) -> type[SQLCompiler]: ...
    def quote_name(self, name: str) -> str: ...
    def regex_lookup(self, lookup_type: str) -> str: ...
    def savepoint_create_sql(self, sid: str) -> str: ...
    def savepoint_commit_sql(self, sid: str) -> str: ...
    def savepoint_rollback_sql(self, sid: str) -> str: ...
    def set_time_zone_sql(self) -> str: ...
    def sql_flush(
        self, style: Any, tables: Sequence[str], *, reset_sequences: bool = False, allow_cascade: bool = False
    ) -> list[str]: ...
    def execute_sql_flush(self, sql_list: Iterable[str]) -> None: ...
    def sequence_reset_by_name_sql(self, style: Style | None, sequences: list[Any]) -> list[Any]: ...
    def sequence_reset_sql(self, style: Style, model_list: Sequence[type[Model]]) -> list[Any]: ...
    def start_transaction_sql(self) -> str: ...
    def end_transaction_sql(self, success: bool = True) -> str: ...
    def tablespace_sql(self, tablespace: str | None, inline: bool = False) -> str: ...
    def prep_for_like_query(self, x: str) -> str: ...
    prep_for_iexact_query: Any
    def validate_autopk_value(self, value: int) -> int: ...
    def adapt_unknown_value(self, value: Any) -> Any: ...
    def adapt_integerfield_value(self, value: Any, internal_type: Any) -> Any: ...
    def adapt_datefield_value(self, value: date | None) -> str | None: ...
    def adapt_datetimefield_value(self, value: real_datetime | None) -> str | None: ...
    def adapt_timefield_value(self, value: real_datetime | time | None) -> str | None: ...
    def adapt_decimalfield_value(
        self, value: Decimal | None, max_digits: int | None = None, decimal_places: int | None = None
    ) -> str | None: ...
    def adapt_ipaddressfield_value(self, value: str | None) -> str | None: ...
    def adapt_json_value(self, value: Any, encoder: type[json.JSONEncoder] | None) -> str: ...
    def year_lookup_bounds_for_date_field(self, value: int, iso_year: bool = False) -> list[str]: ...
    def year_lookup_bounds_for_datetime_field(self, value: int, iso_year: bool = False) -> list[str]: ...
    def get_db_converters(self, expression: Expression) -> list[Any]: ...
    def convert_durationfield_value(
        self, value: float | None, expression: Expression, connection: BaseDatabaseWrapper
    ) -> timedelta | None: ...
    def check_expression_support(self, expression: Any) -> None: ...
    def conditional_expression_supported_in_where_clause(self, expression: Any) -> bool: ...
    def combine_expression(self, connector: str, sub_expressions: list[str]) -> str: ...
    def combine_duration_expression(self, connector: Any, sub_expressions: Any) -> str: ...
    def binary_placeholder_sql(self, value: Case | None) -> str: ...
    def modify_insert_params(self, placeholder: str, params: Any) -> Any: ...
    def integer_field_range(self, internal_type: Any) -> tuple[int, int]: ...
    def subtract_temporals(self, internal_type: Any, lhs: Any, rhs: Any) -> tuple[str, tuple[Any, ...]]: ...
    def window_frame_value(self, value: int | None) -> str: ...
    def window_frame_rows_start_end(self, start: int | None = None, end: int | None = None) -> tuple[str, str]: ...
    def window_frame_range_start_end(self, start: int | None = None, end: int | None = None) -> tuple[str, str]: ...
    def explain_query_prefix(self, format: str | None = None, **options: Any) -> str: ...
    def insert_statement(self, on_conflict: OnConflict | None = None) -> str: ...
    def on_conflict_suffix_sql(
        self, fields: Any, on_conflict: Any, update_fields: Any, unique_fields: Any
    ) -> str | Any: ...
    def prepare_join_on_clause(
        self, lhs_table: str, lhs_field: Field, rhs_table: str, rhs_field: Field
    ) -> tuple[Col, Col]: ...
    def format_debug_sql(self, sql: str) -> str: ...
    def format_json_path_numeric_index(self, num: int) -> str: ...
    def compile_json_path(self, key_transforms: Iterable[str], include_root: bool = True) -> str: ...



from collections.abc import Callable, Sequence
from contextlib import AbstractContextManager
from types import TracebackType
from typing import Any, TypeAlias, TypeVar

from django.test import LiveServerTestCase
from typing_extensions import Self

_F = TypeVar("_F", bound=Callable[..., Any])

_WebDriver: TypeAlias = Any
_Options: TypeAlias = Any

class SeleniumTestCaseBase(type):
    browsers: Any
    selenium_hub: str | None = None
    external_host: str | None = None
    browser: Any
    headless: bool = False
    def __new__(cls, name: str, bases: tuple[type, ...], attrs: dict[str, Any]) -> SeleniumTestCaseBase: ...
    @classmethod
    def import_webdriver(cls, browser: Any) -> type[_WebDriver]: ...
    @classmethod
    def import_options(cls, browser: Any) -> type[_Options]: ...
    @classmethod
    def get_capability(cls, browser: Any) -> dict[str, Any]: ...
    def create_options(self) -> _Options: ...
    def create_webdriver(self) -> _WebDriver: ...

class ChangeWindowSize:
    def __init__(self, width: int, height: int, selenium: Any) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        traceback: TracebackType | None,
    ) -> None: ...

class SeleniumTestCase(LiveServerTestCase, metaclass=SeleniumTestCaseBase):
    tags: set[str]
    implicit_wait: int
    external_host: str | None = None
    screenshots: bool = False
    selenium: Any
    def desktop_size(self) -> AbstractContextManager[None]: ...
    def small_screen_size(self) -> AbstractContextManager[None]: ...
    def mobile_size(self) -> AbstractContextManager[None]: ...
    def rtl(self) -> AbstractContextManager[None]: ...
    def dark(self) -> AbstractContextManager[None]: ...
    def set_emulated_media(self, *, media: str | None = None, features: list[dict[str, str]] | None = None) -> None: ...
    def high_contrast(self) -> AbstractContextManager[None]: ...
    def take_screenshot(self, name: str) -> None: ...
    def get_browser_logs(self, source: str | None = None, level: str = "ALL") -> list[dict[str, Any]]: ...
    def disable_implicit_wait(self) -> AbstractContextManager[None]: ...

def screenshot_cases(method_names: str | Sequence[str]) -> Callable[[_F], _F]: ...



from typing import Any

from django.db import models
from typing_extensions import override

class AreaField(models.FloatField):
    geo_field: Any
    def __init__(self, geo_field: Any) -> None: ...
    @override
    def get_prep_value(self, value: Any) -> Any: ...
    @override
    def get_db_prep_value(self, value: Any, connection: Any, prepared: bool = ...) -> Any: ...
    def from_db_value(self, value: Any, expression: Any, connection: Any) -> Any: ...
    @override
    def get_internal_type(self) -> Any: ...

class DistanceField(models.FloatField):
    geo_field: Any
    def __init__(self, geo_field: Any) -> None: ...
    @override
    def get_prep_value(self, value: Any) -> Any: ...
    @override
    def get_db_prep_value(self, value: Any, connection: Any, prepared: bool = ...) -> Any: ...
    def from_db_value(self, value: Any, expression: Any, connection: Any) -> Any: ...
    @override
    def get_internal_type(self) -> Any: ...



import datetime
from collections.abc import Awaitable, Callable, Iterable, Iterator, Mapping, Sequence
from re import Pattern
from typing import Any, BinaryIO, Literal, NoReturn, Protocol, TypeAlias, TypeVar, overload, type_check_only

from django.contrib.auth.models import _AnyUser
from django.contrib.sessions.backends.base import SessionBase
from django.contrib.sites.models import Site
from django.core.files import uploadedfile, uploadhandler
from django.urls import ResolverMatch
from django.utils.datastructures import CaseInsensitiveMapping, ImmutableList, MultiValueDict
from django.utils.functional import cached_property
from typing_extensions import Self, override

RAISE_ERROR: object
host_validation_re: Pattern[str]

@type_check_only
class _PostDataProtocol(Protocol):
    def read(self, num_bytes: int | None = ..., /) -> bytes: ...
    def readline(self, limit: int | None = ..., /) -> bytes: ...
    def readlines(self) -> list[bytes]: ...
    def __iter__(self) -> Iterator[bytes]: ...

class UnreadablePostError(OSError): ...
class RawPostDataException(Exception): ...

_UploadHandlerList: TypeAlias = list[uploadhandler.FileUploadHandler] | ImmutableList[uploadhandler.FileUploadHandler]

class HttpHeaders(CaseInsensitiveMapping[str]):
    HTTP_PREFIX: str
    UNPREFIXED_HEADERS: set[str]
    def __init__(self, environ: Mapping[str, Any]) -> None: ...
    @classmethod
    def parse_header_name(cls, header: str) -> str | None: ...
    @classmethod
    def to_wsgi_name(cls, header: str) -> str: ...
    @classmethod
    def to_asgi_name(cls, header: str) -> str: ...
    @classmethod
    def to_wsgi_names(cls, headers: Mapping[str, Any]) -> dict[str, Any]: ...
    @classmethod
    def to_asgi_names(cls, headers: Mapping[str, Any]) -> dict[str, Any]: ...

class HttpRequest:
    GET: _ImmutableQueryDict
    POST: _ImmutableQueryDict
    COOKIES: dict[str, str]
    META: dict[str, Any]
    FILES: MultiValueDict[str, uploadedfile.UploadedFile]
    path: str
    path_info: str
    method: str | None
    resolver_match: ResolverMatch | None
    content_type: str | None
    content_params: dict[str, str] | None
    _body: bytes
    _stream: BinaryIO
    # Attributes added by optional parts of Django
    # django.contrib.admin views:
    current_app: str
    # django.contrib.auth.middleware.AuthenticationMiddleware:
    user: _AnyUser
    # django.contrib.auth.middleware.AuthenticationMiddleware:
    auser: Callable[[], Awaitable[_AnyUser]]
    # django.middleware.locale.LocaleMiddleware:
    LANGUAGE_CODE: str
    # django.contrib.sites.middleware.CurrentSiteMiddleware
    site: Site
    # django.contrib.sessions.middleware.SessionMiddleware
    session: SessionBase
    # The magic. If we instantiate HttpRequest directly somewhere, it has
    # mutable GET and POST. However, both ASGIRequest and WSGIRequest have immutable,
    # so when we use HttpRequest to refer to any of them we want exactly this.
    # Case when some function creates *exactly* HttpRequest (not subclass)
    # remain uncovered, however it's probably the best solution we can afford.
    def __new__(cls) -> _MutableHttpRequest: ...
    # When both __init__ and __new__ are present, mypy will prefer __init__
    # (see comments in mypy.checkmember.type_object_type)
    # def __init__(self) -> None: ...
    def get_host(self) -> str: ...
    def get_port(self) -> str: ...
    def get_full_path(self, force_append_slash: bool = ...) -> str: ...
    def get_full_path_info(self, force_append_slash: bool = ...) -> str: ...
    def get_signed_cookie(
        self, key: str, default: Any = ..., salt: str = ..., max_age: int | datetime.timedelta | None = ...
    ) -> str | None: ...
    def build_absolute_uri(self, location: str | None = ...) -> str: ...
    @property
    def scheme(self) -> str | None: ...
    def is_secure(self) -> bool: ...
    @property
    def encoding(self) -> str | None: ...
    @encoding.setter
    def encoding(self, val: str) -> None: ...
    @property
    def upload_handlers(self) -> _UploadHandlerList: ...
    @upload_handlers.setter
    def upload_handlers(self, upload_handlers: _UploadHandlerList) -> None: ...
    @cached_property
    def accepted_types(self) -> list[MediaType]: ...
    @cached_property
    def accepted_types_by_precedence(self) -> list[MediaType]: ...
    def accepted_type(self, media_type: str) -> MediaType | None: ...
    def get_preferred_type(self, media_types: Sequence[str]) -> str | None: ...
    def parse_file_upload(
        self, META: Mapping[str, Any], post_data: _PostDataProtocol
    ) -> tuple[QueryDict, MultiValueDict[str, uploadedfile.UploadedFile]]: ...
    @cached_property
    def headers(self) -> HttpHeaders: ...
    @property
    def body(self) -> bytes: ...
    def _load_post_and_files(self) -> None: ...
    def accepts(self, media_type: str) -> bool: ...
    def close(self) -> None: ...
    # File-like and iterator interface, a minimal subset of BytesIO.
    def read(self, n: int | None = -1, /) -> bytes: ...
    def readline(self, limit: int | None = -1, /) -> bytes: ...
    def __iter__(self) -> Iterator[bytes]: ...
    def readlines(self) -> list[bytes]: ...

@type_check_only
class _MutableHttpRequest(HttpRequest):
    GET: QueryDict  # type: ignore[assignment]
    POST: QueryDict  # type: ignore[assignment]

_Z = TypeVar("_Z")

# mypy uses mro to pick between `__init__` and `__new__` for return types and will prefers `__init__` over `__new__`
# in case of a tie. So to be able to specialize type via `__new__` (which is required for pyright to work),
# we need to use an intermediary class for the `__init__` method.
# See https://github.com/python/mypy/issues/17251
# https://github.com/python/mypy/blob/c724a6a806655f94d0c705a7121e3d671eced96d/mypy/typeops.py#L148-L149
@type_check_only
class _QueryDictMixin:
    def __init__(
        self,
        query_string: str | bytes | None = ...,
        mutable: bool = ...,
        encoding: str | None = ...,
    ) -> None: ...

class QueryDict(_QueryDictMixin, MultiValueDict[str, str]):
    _mutable: bool
    # We can make it mutable only by specifying `mutable=True`.
    # It can be done a) with kwarg and b) with pos. arg. `overload` has
    # some problems with args/kwargs + Literal, so two signatures are required.
    # ('querystring', True, [...])
    @overload
    def __new__(
        cls,
        query_string: str | bytes | None,
        mutable: Literal[True],
        encoding: str | None = ...,
    ) -> QueryDict: ...
    # ([querystring='string',] mutable=True, [...])
    @overload
    def __new__(
        cls,
        *,
        query_string: str | bytes | None = ...,
        mutable: Literal[True],
        encoding: str | None = ...,
    ) -> QueryDict: ...
    # Otherwise it's immutable
    @overload
    def __new__(
        cls,
        query_string: str | bytes | None = ...,
        mutable: Literal[False] = ...,
        encoding: str | None = ...,
    ) -> _ImmutableQueryDict: ...
    @classmethod
    @override
    def fromkeys(  # type: ignore[override]
        cls,
        iterable: Iterable[bytes | str],
        value: str | bytes = ...,
        mutable: bool = ...,
        encoding: str | None = ...,
    ) -> Self: ...
    @property
    def encoding(self) -> str: ...
    @encoding.setter
    def encoding(self, value: str) -> None: ...
    @override
    def __setitem__(self, key: str | bytes, value: str | bytes) -> None: ...
    @override
    def __delitem__(self, key: str | bytes) -> None: ...
    @override
    def setlist(self, key: str | bytes, list_: Iterable[str | bytes]) -> None: ...
    @override
    def setlistdefault(self, key: str | bytes, default_list: list[str] | None = ...) -> list[str]: ...
    @override
    def appendlist(self, key: str | bytes, value: str | bytes) -> None: ...
    # Fake signature (because *args is used in source, but it fails with more that 1 argument)
    @overload  # type:ignore[override]
    def pop(self, key: str | bytes, /) -> list[str]: ...
    @overload
    @override
    def pop(self, key: str | bytes, default: str | _Z = ..., /) -> list[str] | _Z: ...
    @override
    def popitem(self) -> tuple[str, list[str]]: ...  # type:ignore[override]
    @override
    def clear(self) -> None: ...
    @override
    def setdefault(self, key: str | bytes, default: str | bytes | None = ...) -> str: ...
    @override
    def copy(self) -> QueryDict: ...
    def urlencode(self, safe: str | None = ...) -> str: ...

@type_check_only
class _ImmutableQueryDict(QueryDict):
    _mutable: Literal[False]
    @override
    def __setitem__(self, key: str | bytes, value: str | bytes) -> NoReturn: ...
    @override
    def __delitem__(self, key: str | bytes) -> NoReturn: ...
    @override
    def setlist(self, key: str | bytes, list_: Iterable[str | bytes]) -> NoReturn: ...
    @override
    def setlistdefault(self, key: str | bytes, default_list: list[str] | None = ...) -> NoReturn: ...
    @override
    def appendlist(self, key: str | bytes, value: str | bytes) -> NoReturn: ...
    # Fake signature (because *args is used in source, but it fails with more that 1 argument)
    @overload
    @override
    def pop(self, key: str | bytes, /) -> NoReturn: ...
    @overload
    @override
    def pop(self, key: str | bytes, default: str | _Z = ..., /) -> NoReturn: ...
    @override
    def popitem(self) -> NoReturn: ...
    @override
    def clear(self) -> NoReturn: ...
    @override
    def setdefault(self, key: str | bytes, default: str | bytes | None = ...) -> NoReturn: ...
    @override
    def copy(self) -> QueryDict: ...  # type: ignore[override]
    @override
    def urlencode(self, safe: str | None = ...) -> str: ...
    # Fakes for convenience (for `request.GET` and `request.POST`). If dict
    # was created by Django, there is no chance to hit `List[object]` (empty list)
    # edge case.
    @override
    def __getitem__(self, key: str) -> str: ...
    @override
    def dict(self) -> dict[str, str]: ...  # type: ignore[override]

class MediaType:
    main_type: str
    sub_type: str
    params: dict[str, bytes]
    def __init__(self, media_type_raw_line: str) -> None: ...
    def match(self, other: str) -> bool: ...
    @cached_property
    def quality(self) -> float: ...
    @property
    def specificity(self) -> int: ...
    @cached_property
    def range_params(self) -> dict[str, bytes]: ...

@overload
def bytes_to_text(s: None, encoding: str) -> None: ...
@overload
def bytes_to_text(s: bytes | str, encoding: str) -> str: ...
def split_domain_port(host: str) -> tuple[str, str]: ...
def validate_host(host: str, allowed_hosts: Iterable[str]) -> bool: ...



from django.contrib.gis.db.backends.base.features import BaseSpatialFeatures
from django.db.backends.oracle.features import DatabaseFeatures as OracleDatabaseFeatures
from django.utils.functional import cached_property
from typing_extensions import override

class DatabaseFeatures(BaseSpatialFeatures, OracleDatabaseFeatures):
    supports_add_srs_entry: bool
    supports_geometry_field_introspection: bool
    supports_geometry_field_unique_index: bool
    supports_perimeter_geodetic: bool
    supports_dwithin_distance_expr: bool
    supports_tolerance_parameter: bool

    @cached_property
    @override
    def django_test_skips(self) -> dict[str, set[str]]: ...  # type: ignore[override]
